// device file for
// HP Digitizing Oscilloscope 54616B (!)
// (c) 2000 R. Schumann & M. Dammasch
//
//M. Dammasch 05.11.1998 upgrade to v2.0beta
//M. Dammasch 14.12.1998 error()-handling changed (only 0 returns)
//M. Dammasch 20.01.2000 modified for HP 54616B !!!
//M. Dammasch 24.01.2000 added trace-data readout support
//
// last edit: M. Dammasch 21.01.2000

//this header has always to be included !!
#include <list.h>
#include <sched.h>
#include <error.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <gpiberr.h>
#include <ib.h>

#define HP_REFERENCE_LEFT    -1
#define HP_REFERENCE_CENTER  0
#define HP_REFERENCE_RIGHT   1

#define HP_TYPE_REALTIME     0
#define HP_TYPE_NORMAL       0
#define HP_TYPE_SINGLE       0
#define HP_TYPE_REPETITIVE   1
#define HP_TYPE_AVERAGE      1
#define HP_TYPE_TRIGGERED    1
#define HP_TYPE_PEAK         2
#define HP_TYPE_TRACE        3

#define HP_FORMAT_COMPRESSED  1
#define HP_FORMAT_BYTE        1
#define HP_FORMAT_WORD        2
#define HP_FORMAT_ASCII       0
#define HP_FORMAT_TRACE_ASCII 3

#define HP_STRING_MAX  40

extern int ibsta, ibcnt, iberr;

static int gpib_adress;    // Gpib-adress of the oszi
static char cmd[HP_STRING_MAX+1];
static int TLAM;
static int Channel;        // Channel to digitize
static char errmsg[51];    // local error message string
static char qterrmsg[81];  // compound error message string for qt-error()-widget

static int slowmode;

static int pc_set;

static int tb_type;
static double tb_range;
static double tb_delay;
static int tb_delref;

static double yrange;
static double yoffset;

static double dig_del;
static int aq_type;
static int aq_format;
static int aq_counts;
static int aq_points;
static int aq_complete;
static int aq_scroff;

static void cls_gpib(void)
{
  printf("\nclearing HP device...");
  if( ibclr(gpib_adress) & ERR)
    gpiberr("clr err");
}

/*************************************************/
static int HP_wrtcmd ( char *cmdstr )
{
  //ibcnt = 0;
  if ( ibwrt(gpib_adress,cmdstr,strlen(cmdstr)) & ERR)
  {
    gpiberr("wrt err");
    cls_gpib();
    sprintf(errmsg,"\tHP ibwrite err:\nCouldn't send <%s>\n",cmdstr);
    return 0;
  }
  //printf("\nHP-W(%d/%d):<%s>",ibcnt,strlen(cmdstr),cmdstr);
  return 1;
}

static int HP_rdstr ( char *retstr )
{
  ibcnt = 0;
  if ( ibrd(gpib_adress,retstr,HP_STRING_MAX) & ERR)
  {
    gpiberr("read error");
    cls_gpib();
    sprintf(errmsg,"\tHP ibread err:\nReceived <%s>\n",retstr);
    return 0;
  }
  if (ibcnt <= HP_STRING_MAX)
    retstr[ibcnt] = '\0';

  //printf("\nHP-R(%d):<%s>",ibcnt,retstr);
  return 1;
}
///////////////////////////////////////////////////
static int HP_set_timebase ( int type, double range, double delay, int reference )
{
  char cmd[HP_STRING_MAX+1];

  sprintf(cmd,":timebase:mode normal\n");
  if (!HP_wrtcmd(cmd)) return 0;
 
  /*
  if (type == HP_TYPE_REALTIME) sprintf(cmd,":timebase:mode single\n");
  else
  if (type == HP_TYPE_REPETITIVE) sprintf(cmd,":timebase:mode triggered\n");
  else return 0;
  if (!HP_wrtcmd(cmd)) return 0;
  */

  sprintf(cmd,":timebase:range %E",range);
  if (!HP_wrtcmd(cmd)) return 0;

  sprintf(cmd,":timebase:delay %E",delay);
  if (!HP_wrtcmd(cmd)) return 0;

  switch (reference)
  {
    case HP_REFERENCE_LEFT :      
      sprintf(cmd,":timebase:reference left\n");
    break;
    case HP_REFERENCE_CENTER :
      sprintf(cmd,":timebase:reference center\n");
    break;
    /*
    case HP_REFERENCE_RIGHT :
      sprintf(cmd,":timebase:reference right\n");
    break;
    */
    default :    
      sprintf(errmsg,"\nIllegal reference (%d)\n",reference);
      return 0;
    break;
  }
  if (!HP_wrtcmd(cmd)) return 0;
  
  return 1;
}

static int HP_get_timebase ( int *type, double *range, double *delay, int *reference )
{
  char cmd[HP_STRING_MAX+1], str[HP_STRING_MAX+1];

  /*
  sprintf(cmd,":timebase:sample?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0; 
  if (strstr(str,"REAL") != NULL) *type = HP_TYPE_REALTIME;
  else
    if (strstr(str,"REP") != NULL) *type = HP_TYPE_REPETITIVE;
  else
  {
    sprintf(errmsg,"\nIllegal timebase sample type (%s)\n",str);
    return 0;
  }
  */
  *type = HP_TYPE_REPETITIVE;

  sprintf(cmd,":timebase:range?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *range = atof(str);

  sprintf(cmd,":timebase:delay?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *delay = atof(str);

  sprintf(cmd,":timebase:reference?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;

  if (strstr(str,"LEFT") != NULL) *reference = HP_REFERENCE_LEFT;
  else
    if (strstr(str,"CENT") != NULL) *reference = HP_REFERENCE_CENTER;
  else
    if (strstr(str,"RIGH") != NULL) *reference = HP_REFERENCE_RIGHT;
  else
  {
    sprintf(errmsg,"\nIllegal reference (%s)\n",str);
    return 0;
  }
  
  return 1;
}

static int HP_set_channel ( int channel, double range, double offset )
{
  char cmd[HP_STRING_MAX+1];
  
  sprintf(cmd,":channel%d:range %E",channel,range);
  if (!HP_wrtcmd(cmd)) return 0;

  sprintf(cmd,":channel%d:offset %E",channel,offset);
  if (!HP_wrtcmd(cmd)) return 0;

  return 1;
}

static int HP_get_channel ( int channel, double *range, double *offset )
{
  char cmd[HP_STRING_MAX+1], str[HP_STRING_MAX+1];
  
  sprintf(cmd,":channel%d:range?",channel);
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *range = atof(str);

  sprintf(cmd,":channel%d:offset?",channel);
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *offset = atof(str);

  return 1;
}

static int HP_set_acquire ( int type, int format, int points, int counts, int complete )
{
  char cmd[HP_STRING_MAX+1];
  
  sprintf(cmd,":timebase:mode normal\n");
  if (!HP_wrtcmd(cmd)) return 0;

  switch (type)
  {
    case HP_TYPE_NORMAL :
      sprintf(cmd,":acquire:type normal\n");
      if (!HP_wrtcmd(cmd)) return 0;
      sprintf(cmd,":waveform:points %d",points);
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    case HP_TYPE_AVERAGE :
      sprintf(cmd,":acquire:type average\n");
      if (!HP_wrtcmd(cmd)) return 0;
      sprintf(cmd,":waveform:points %d",5000);
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    case HP_TYPE_PEAK :
      sprintf(cmd,":acquire:type peak\n");
      if (!HP_wrtcmd(cmd)) return 0;
      sprintf(cmd,":waveform:points %d",1000);
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    default :
      sprintf(errmsg,"\nIllegal type (%d)\n",type);
      return 0;
    break;
  }
 
  switch (format)
  {
    case HP_FORMAT_BYTE :
      sprintf(cmd,":waveform:format byte\n");
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    case HP_FORMAT_WORD :
      sprintf(cmd,":waveform:format word\n");
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    case HP_FORMAT_ASCII :
      sprintf(cmd,":waveform:format asc\n");
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    default :
      sprintf(errmsg,"\nIllegal format (%d)\n",format);
      return 0;
    break; 
  }

  sprintf(cmd,":acquire:count %d",counts);
  if (!HP_wrtcmd(cmd)) return 0;
  sprintf(cmd,":acquire:complete %d",complete);
  if (!HP_wrtcmd(cmd)) return 0;

  return 1;
}

static int HP_get_acquire ( int *type, int *format, int *points, int *counts, int *complete )
{
  char cmd[HP_STRING_MAX+1], str[HP_STRING_MAX+1];
  
  sprintf(cmd,":acquire:type?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  if (strstr(str,"NORM") != NULL) *type = HP_TYPE_NORMAL;
  else
    if (strstr(str,"AVER") != NULL) *type = HP_TYPE_AVERAGE;
  else
    if (strstr(str,"PEAK") != NULL) *type = HP_TYPE_PEAK;
  else
  {
    sprintf(errmsg,"\nIllegal type (%s)\n",str);
    return 0;
  }

  sprintf(cmd,":waveform:format?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  if (strstr(str,"BYTE") != NULL) *format = HP_FORMAT_BYTE;
  else
    if (strstr(str,"WORD") != NULL) *format = HP_FORMAT_WORD;
  else
    if (strstr(str,"ASC") != NULL) *format = HP_FORMAT_ASCII;
  else
  {
    sprintf(errmsg,"\nIllegal format (%s)\n",str);
    return 0;
  }

  sprintf(cmd,":acquire:points?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *points = atoi(str);

  sprintf(cmd,":acquire:count?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *counts = atoi(str);

  sprintf(cmd,":acquire:complete?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *complete = atoi(str);

  return 1;
}

static int HP_opc ( void )
{
  char quest[HP_STRING_MAX+1], answ[HP_STRING_MAX+1];

  sprintf(quest,"*OPC?\n");
  if (!HP_wrtcmd(quest)) return 0;
  if (!HP_rdstr(answ)) return 0;
  if (atoi(answ) == 1) return 1;
  return 0;
  
}

static int HP_ter ( void )
{
  char quest[HP_STRING_MAX+1], answ[HP_STRING_MAX+1];

  sprintf(quest,":TER?\n");
  if (!HP_wrtcmd(quest)) return 0;
  if (!HP_rdstr(answ)) return 0;
  if (atoi(answ) == 1) return 1;
  return 0;
  
}
/*************************************************/

//init function for HP-Scope
//called before meausurement starts
int HP_init(List *device_data, Scheduler *parameters)
{
  //initial HP
  
  printf("\n\n***** Connecting GPIB Device 'HP' *****");
  if((gpib_adress = ibfind("HP")) & ERR){
    gpiberr("ibfind err");
    if (!parameters->error(E_GPIB," HP ibfind err"))
      return 0;   //return FALSE -> measurement stops
  }
  printf("\nHP GPIB-Address is set to #%d ",gpib_adress);
    
  printf("\nsend IFC "); //ibsic aus Lib. 
  if( ibsic(gpib_adress) & ERR){
    gpiberr("ibsic err");
    cls_gpib();
    if (!parameters->error(E_GPIB," HP ibsic err"))
      return 0;   //return FALSE -> measurement stops
  }
  printf(" done");

  printf("\nsend REM ");
  if( ibsre(gpib_adress,1) & ERR){
    gpiberr("ibsre err");
    cls_gpib();
    if (!parameters->error(E_GPIB," HP ibsre err"))
      return 0;   //return FALSE -> measurement stops
  }
  printf(" done");

  cls_gpib();

  printf("\n***** Completed Successfully *****\n");
  
  sprintf(errmsg,"\tNo error(s) occured\n");

  sprintf( cmd, "*IDN?\n");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(cmd)) return 0;
  printf("\nDevice-ID: %s ",cmd);
  
  device_data->getvar(&TLAM,"all","timeout");
  device_data->getvar(&Channel,"HP","Channel");
  device_data->getvar(&aq_type,"HP","acq_type");  
  device_data->getvar(&aq_format,"HP","acq_format");
  device_data->getvar(&aq_counts,"HP","acq_count");  
  device_data->getvar(&aq_points,"HP","acq_point");
  device_data->getvar(&aq_complete,"HP","acq_complete");    
  device_data->getvar(&aq_scroff,"HP","acq_scr_off");  
  device_data->getvar(&pc_set,"HP","pc_set_parameter");
  device_data->getvar(&tb_type,"HP","tb_type");  
  device_data->getvar(&tb_range,"HP","tb_range");  
  device_data->getvar(&tb_delay,"HP","tb_delay");  
  device_data->getvar(&tb_delref,"HP","tb_delay_ref");  
  device_data->getvar(&yrange,"HP","yrange");  
  device_data->getvar(&yoffset,"HP","yoffset");  

  device_data->getvar(&dig_del,"HP","digitize_delay");
  if (dig_del > 0) slowmode = 1;
  else slowmode = 0;

  if (aq_type != HP_TYPE_TRACE) {
    if (Channel < 1 || Channel > 2) Channel = 1;
  }
  else {
    if (Channel < 1 || Channel > 100) Channel = 1;
  }
  
  printf("\ninitializing HP...");

  /*
  sprintf( cmd, "*cls\n"); //Clear
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
    gpiberr("wrt err");
    cls_gpib();
    if (!parameters->error(E_GPIB," HP ibwrite err "))
      return 0;   //return FALSE -> measurement stops
  }
  */
 
  if (aq_type != HP_TYPE_AVERAGE)
  {
    sprintf( cmd, ":stop\n");
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
    {
      gpiberr("wrt err");
      cls_gpib();
      if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
	return 0;   //return FALSE -> measurement stops
    } 
    sprintf( cmd, ":display:connect off\n");
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
    {
      gpiberr("wrt err");
      cls_gpib();
      if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
	return 0;   //return FALSE -> measurement stops
    } 
  }

  /*
  sprintf( cmd, ":system:longform off;header off\n");
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
  {
    gpiberr("wrt err");
    cls_gpib();
    if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
      return 0;   //return FALSE -> measurement stops
  } 
  */

  if (aq_type != HP_TYPE_TRACE) {
    sprintf( cmd, ":waveform:source channel%d",Channel);
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
      {
	gpiberr("wrt err");
	cls_gpib();
	if (!parameters->error(E_IB_WRITE," HP ibwrite err"))
	  return 0;   //return FALSE -> measurement stops
      }
  }

  if (aq_type != HP_TYPE_TRACE && pc_set)  // pc_set: set all parameters via GPIB
  {
    if (HP_set_timebase(tb_type,tb_range,tb_delay,tb_delref) != 1)
    {
      sprintf(qterrmsg,"\nHP_SET_TIMEBASE(): Error due to\n%s",errmsg);
      if (!parameters->error(E_GPIB,qterrmsg))
	return 0;
    }  
    if (HP_set_channel(Channel,yrange,yoffset) != 1)
    {
      sprintf(qterrmsg,"\nHP_SET_CHANNEL(): Error due to\n%s",errmsg);
      if (!parameters->error(E_GPIB,qterrmsg))
	return 0;
    }   
    if (HP_set_acquire(aq_type,aq_format,aq_points,aq_counts,aq_complete) != 1)
    {
      sprintf(qterrmsg,"\nHP_SET_ACQUIRE(): Error due to\n%s",errmsg);
      if (!parameters->error(E_GPIB,qterrmsg))
	return 0;
    } 
  }
  else if (aq_type != HP_TYPE_TRACE)
  {
    if (HP_set_acquire(aq_type,aq_format,aq_points,aq_counts,aq_complete) != 1)
    {
      sprintf(qterrmsg,"\nHP_SET_ACQUIRE(): Error due to\n%s",errmsg);
      if (!parameters->error(E_GPIB,qterrmsg))
	return 0;
    } 

    if (HP_get_timebase(&tb_type,&tb_range,&tb_delay,&tb_delref) != 1)
    {
      sprintf(qterrmsg,"\nHP_GET_TIMEBASE(): Error due to\n%s",errmsg);
      if (!parameters->error(E_GPIB,qterrmsg))
	return 0;
    }  
    if (HP_get_channel(Channel,&yrange,&yoffset) != 1)
    {
      sprintf(qterrmsg,"\nHP_GET_CHANNEL(): Error due to\n%s",errmsg);
      if (!parameters->error(E_GPIB,qterrmsg))
	return 0;
    }   
    if (HP_get_acquire(&aq_type,&aq_format,&aq_points,&aq_counts,&aq_complete) != 1)
    {
      sprintf(qterrmsg,"\nHP_GET_ACQUIRE(): Error due to\n%s",errmsg);
      if (!parameters->error(E_GPIB,qterrmsg))
	return 0;
    }
  }
  
  if (aq_type == HP_TYPE_AVERAGE) {
    sprintf( cmd, ":waveform:points?\n"); // number of transmitted bytes (comp format only!)
    if (!HP_wrtcmd(cmd)) return 0;
    if (!HP_rdstr(cmd)) return 0;
    if (aq_points != atoi(cmd)) {
      sprintf(qterrmsg,"HP_INIT(): Different values for AQ_POINTS (%d) and WAV_POINTS (%d)",aq_points,atoi(cmd));
      if (!parameters->error(E_GPIB,qterrmsg))
	return 0;
    }  
  }

  /*
  if (aq_scroff)
  {
    sprintf( cmd, ":display:screen off\n");
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
    {
      gpiberr("wrt err");
      cls_gpib();
      if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
	return 0;   //return FALSE -> measurement stops
    }
  } 
  */

  printf("\b\b\b done");
  return 1;
}


int HP_close(List *device_data, Scheduler *parameters)
{
  printf("closing HP\n");

  if (aq_type != HP_TYPE_TRACE) {
    if (HP_get_timebase(&tb_type,&tb_range,&tb_delay,&tb_delref) != 1)
      {
	sprintf(qterrmsg,"\nHP_GET_TIMEBASE(): Error due to\n%s",errmsg);
	if (!parameters->error(E_GPIB,qterrmsg))
	  return 0;
      }  
    if (HP_get_channel(Channel,&yrange,&yoffset) != 1)
      {
	sprintf(qterrmsg,"\nHP_GET_CHANNEL(): Error due to\n%s",errmsg);
	if (!parameters->error(E_GPIB,qterrmsg))
	  return 0;
      }   
    if (HP_get_acquire(&aq_type,&aq_format,&aq_points,&aq_counts,&aq_complete) != 1)
      {
	sprintf(qterrmsg,"\nHP_GET_ACQUIRE(): Error due to\n%s",errmsg);
	if (!parameters->error(E_GPIB,qterrmsg))
	  return 0;
      }     
  }

  device_data->putvar(Channel,"HP","Channel");
  device_data->putvar(tb_type,"HP","tb_type");  
  device_data->putvar(tb_range,"HP","tb_range");  
  device_data->putvar(tb_delay,"HP","tb_delay");  
  device_data->putvar(tb_delref,"HP","tb_delay_ref");  
  device_data->putvar(yrange,"HP","yrange");  
  device_data->putvar(yoffset,"HP","yoffset");  
  device_data->putvar(aq_type,"HP","acq_type");  
  device_data->putvar(aq_format,"HP","acq_format");  
  device_data->putvar(aq_counts,"HP","acq_count");  
  device_data->putvar(aq_points,"HP","acq_point");  
  device_data->putvar(aq_complete,"HP","acq_complete");  
  device_data->putvar(dig_del,"HP","digitize_delay");
  
  return 1; //function has to reutrn a vlue -> >1 otherwise measurement stops
}


//pre-trigger device function for HP
int HP_pre(List *device_data, Scheduler *parameters) 
{  
  /*
   * Send 'digitize' / 'run' command to read the raw waveform data 
   */

  switch (aq_type)
  {
    case HP_TYPE_NORMAL :
      printf("\nacquiring data, please wait");
      sprintf( cmd, ":run\n");
      if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
	gpiberr("wrt err");
	cls_gpib();
	if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
	  return 0;   //return FALSE -> measurement stops
      }   
    break;
    case HP_TYPE_PEAK :
      printf("\nacquiring data in PEAK mode , please wait");
      sprintf( cmd, ":run\n");
      if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
	gpiberr("wrt err");
	cls_gpib();
	if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
	  return 0;   //return FALSE -> measurement stops
      }   
    break;
    case HP_TYPE_AVERAGE :
      printf("\ndigitizing, please wait");
      sprintf( cmd, ":digitize channel%d",Channel);  
      if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
      {
	gpiberr("wrt err");
	cls_gpib(); 
	if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
	  return 0;   //return FALSE -> measurement stops
      }
    break;
    case HP_TYPE_TRACE :
      printf("\ngoing to readout trace channel %d, please wait",Channel);
    break;
    default :
      sprintf(errmsg,"\nIllegal type (%d)\n",aq_type);
      return 0;
    break;
  }

  return 1;
}


//device function for HP
int HP(List *device_data, Scheduler *parameters) 
{
  
  unsigned char *waveform, *tp, token[6];
  char           datahead[11], eostr[2];
  int           *wave, wpmax, datamax, i;

  // wait until digitizing / run completed
  if (aq_type != HP_TYPE_TRACE && slowmode) usleep((long unsigned int)(dig_del*aq_counts*1.0E+6));

  if (aq_type == HP_TYPE_NORMAL || aq_type == HP_TYPE_TRACE) {
    sprintf( cmd, ":stop\n");
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
      gpiberr("wrt err");
      cls_gpib();
      if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
	return 0;   //return FALSE -> measurement stops
    }   
  }

  printf(" done");

  if (aq_type != HP_TYPE_TRACE) {
    sprintf( cmd, ":waveform:data?\n");       //ask for data
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
      gpiberr("wrt err");
      cls_gpib();
      if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
	return 0;   //return FALSE -> measurement stops
    }
  }
  else {
    sprintf( cmd, ":trace:data? %d\n",Channel);       //ask for data
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
      gpiberr("wrt err");
      cls_gpib();
      if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
	return 0;   //return FALSE -> measurement stops
    }
    aq_format = HP_FORMAT_TRACE_ASCII;
  }

  if (aq_format != HP_FORMAT_ASCII)  // only BYTE, WORD & TRACE_ASCII start with header
  {
    for (i = 0; i < 11; datahead[i++] = '\0');
    if( ibrd (gpib_adress, datahead, 10)  & ERR)	// read data header info
    {
      datahead[10] = '\0';
      gpib_error("read error for 'datahead' <%s> ",datahead);
      cls_gpib();
      if (!parameters->error(E_IB_READ,"HP ibread err for 'datahead' "))
	return 0;   //return FALSE -> measurement stops
    }
    datamax = atoi(datahead+2);  // number of waveform data bytes 
  }
  else datamax = 6*aq_points;  // ASCII: 16bit data words ("0"..."65535") sep. by ','

  if ((waveform = (unsigned char *)calloc(datamax,sizeof(unsigned char))) == NULL)
  {
    sprintf(qterrmsg,"\nCouldn't allocate vector waveform (%d uchars)\n",datamax);
    if (!parameters->error(E_SYS_MEM,qterrmsg))
      return 0;
  }

  switch (aq_format)
  {
    case HP_FORMAT_WORD :
      wpmax = datamax/2;
    break;
    case HP_FORMAT_BYTE :
    case HP_FORMAT_TRACE_ASCII :
      wpmax = datamax;
    break;
    case HP_FORMAT_ASCII :
    default :
      wpmax = aq_points;
    break;
  }

  if ((wave = (int *)calloc(wpmax,sizeof(int))) == NULL)
  {
    free(waveform);
    sprintf(qterrmsg,"\nCouldn't allocate vector wave (%d ints)\n",
            wpmax);
    if (!parameters->error(E_SYS_MEM,qterrmsg))
      return 0;
  }

  printf("\nreading HP scope data (%d uchars for %d items)...",datamax,wpmax);  
  if( ibrd (gpib_adress, waveform, datamax)  & ERR)
  {
    waveform[ibcnt] = '\0';
    gpib_error("read error for 'waveform' <%s> ",waveform);
    cls_gpib();
    if (!parameters->error(E_IB_READ,"HP ibread err for 'waveform' "))
      return 0;   //return FALSE -> measurement stops
  }
  
  // convert received GPIB-string 'waveform' into array of ints 'wave'
  switch (aq_format)
  {
    case HP_FORMAT_ASCII :
    case HP_FORMAT_TRACE_ASCII :
      i = 0; 
      while ((tp = strtok(((i==0)?(waveform):((unsigned char)NULL)),",")) != NULL)
      {
        strcpy(token,tp);
        wave[i++] = atoi(token);
      }
    break;
    case HP_FORMAT_WORD :
      for(i = 0; i < wpmax; i++)
        wave[i] = ((int)waveform[2*i] << 8) + (int)waveform[2*i+1];
    break;
    case HP_FORMAT_BYTE :
      for(i = 0; i < wpmax; wave[i] = (int)waveform[i++]);
    break;
  }

  eostr[0] = eostr[1] = '\0';  
  if( ibrd (gpib_adress, eostr, 1)  & ERR)	// read data <terminator>
  {
    gpiberr("read error");
    cls_gpib();
    if (!parameters->error(E_IB_READ,"HP ibread err for 'eostr'"))
      return 0;   //return FALSE -> measurement stops
  }
  
  parameters->store(wave,wpmax);	// save the wave data array
  
  // end up read out process: reset scope
  sprintf( cmd, "*cls\n"); //Clear Memory
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
     gpiberr("wrt err");
    cls_gpib();
    if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
      return 0;   //return FALSE -> measurement stops
  }
  printf("\b\b\b end readout HP\n");
  cls_gpib();

  free(waveform);
  free(wave);

  return 1; 
}
