//device file for the 16 channel 12bit scanning ADC ADS1612
//
// 01.12.1999 implemented on host linuxlab for max2.0beta
//last edit: M. Dammasch, Dez/99

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "list.hh"
#include "sched.hh"
#include "camacs.h"
#include "error.hh"

#undef DEBUG_DEV_ADS1612
//#define DEBUG_DEV_ADS1612

#define ADS1612_ID_CODE 1612
#define ADS1612_CHANMAX 2

// global variables
static int  N, qx, TLAM, shots;

/******************* function(s) common to all channels **********************/
int ads1612_readout ( int channel, Scheduler *parameters )
{
  int  d;
  unsigned int dummy;

  qx = camac(0,channel,N,&dummy);   //readout adc
  d = (int)dummy;

  //parameters->store((parameters->act_channel() % shots == 0)?(d):(-1));  //save measured or no-meas value
  parameters->store((parameters->act_channel() % shots == 0)?((d <=2048?(2048-d):(4096-(d-2048)))):(-1));  //save measured or no-meas value

#ifdef DEBUG_DEV_ADS1612
  printf("\nads1612_%02d: %s data stored (%d)",channel,(parameters->act_channel() % shots == 0)?("real"):("virtual"),d);
#endif

  return 1;
}
/*****************************************************************************/

//init function
//called each time before meausurement starts
int ads1612_init(List *device_data, Scheduler *parameters)
{	
  unsigned int x;
  char errmsg[81];

  if (camac_init() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't initialize device "))
      return 0;

  printf("\ninitializing ads1612");
  device_data->getvar(&N,"ads1612","N");
  device_data->getvar(&TLAM,"all","timeout");
  device_data->getvar(&shots,"ads1612","shots");

  //check if ads1612 really present in CAMAC slot N
  qx = camac(6,0,N,&x);

  if (qx < 3) {
    sprintf(errmsg,"ads1612_init(): qx=%d returned ",qx);
    if (!parameters->error(E_CAMAC,errmsg))
      return 0;   //return FALSE -> measurement stops
  }

  if (x != ADS1612_ID_CODE) {
    sprintf(errmsg,"ads1612_init(): Wrong device ID (%d <> %d) ",x,ADS1612_ID_CODE);
    if (!parameters->error(E_CAMAC,errmsg))
      return 0;
  }

  return 1;
}

int ads1612_pre ( List *device_data, Scheduler *parameters )
{
  // nothing to do...

  return 1;
}

int ads1612 ( List *device_data, Scheduler *parameters )
{
  int i;
  char label[6];

  for (i = 0; i < ADS1612_CHANMAX; i++) {
    sprintf(label,"ch%02d",i);
    if (device_data->enabled("ads1612",label)) {
      if ( ! ads1612_readout(i,parameters))
	return 0;
    }
#ifdef DEBUG_DEV_ADS1612
    else {
      printf("\nads1612_%02d: Not enabled - skipping readout ",i);
    }
#endif
  }

  return 1;
}

int ads1612_close(List *device_data, Scheduler *parameters)
{
  printf("\nclosing ads1612 ");
  if (camac_close() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't close device"))
      return 0;

  return 1; //function has to return a value -> 1 otherwise measurement stops
}


