#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "list.hh"
#include "sched.hh"
#include "error.hh"
#include "camacs.h"

/* 26.04.01 S. Gerlach	: 	camac_slot_position -> N
				camac(...) after init !
*/

#undef DEBUG_DEV_DAC3016

/*
  global variables
*/
double voltage,upper_limit,lower_limit;
int N;


int setvoltage(int slot_position,int channel_number,double voltage)
{
  double increment_per_bit=3.051757813E-4;
  unsigned int bit_value;
  bit_value=(unsigned int) ((voltage+10.0)/increment_per_bit);
  if(camac(16,channel_number,slot_position,&bit_value)!=3)
    {
      printf("Fehler beim Setzen der Spannung %f (%d) auf Channel %i des dsp3016 !\n",voltage, bit_value, channel_number);
    }
  return 1;
}

double readvoltage(int slot_position,int channel_number)
{
  double increment_per_bit=3.051757813E-4;
  double voltage;
  unsigned int bit_value;
  if(camac(0,channel_number,slot_position,&bit_value)!=3)
    {
      printf("Fehler beim Auslesen der Spannung aus Channel %i des dsp3016 !\n",channel_number);
    }
  voltage=((double) bit_value)*increment_per_bit-10.0;
  return voltage;
}



int dsp3016_init(List *device_data, Scheduler *parameters)
{
  char text[81];

  printf("Initialisiere dsp3016...\n");
  unsigned int bit_value;

  if (camac_init() != 1)
    {
      return parameters->error(E_GPIB,"CAMAC ibinit err");
    }

  int enabled = device_data->enabled("dsp3016","Channel1");

  if(enabled) {
      printf("Setze Start-Spannung an Channel 1 des dsp3016 !\n");
      device_data->getvar(&voltage,"dsp3016","Start_Voltage1");
      device_data->getvar(&upper_limit,"dsp3016","Upper_Limit1");
      device_data->getvar(&lower_limit,"dsp3016","Lower_Limit1");
      device_data->getvar(&N,"dsp3016","N");
      setvoltage(N,0,voltage);
    }

  if(camac(3,0,N,&bit_value)!=3) {
      sprintf(text,"Fehler beim Initialisieren des dsp3016: %d <> 3016 !\n",bit_value);
      if (!parameters->error(E_CAMAC,text))
	return 0;
    }

  printf("Modul dsp3016 initialized\n");
  printf("dsp3016 Modul-ID: %i\n",bit_value);
  return 1;
}   

int dsp3016_pre(List *device_data, Scheduler *parameters)
{
  int enabled;

  enabled=device_data->enabled("dsp3016","Channel1");
  if(enabled)
    {
      printf("Setzen der Spannung an Channel 1 des dsp3016...\n");
      setvoltage(N,0,lower_limit+(upper_limit-lower_limit)*(parameters->act_channel()-1)/parameters->maximal_channels());
    }
  return 1;
}

int dsp3016(List *device_data, Scheduler *parameters)
{
  int enabled;

  enabled=device_data->enabled("dsp3016","Channel1");
  if(enabled)
    {
      printf("Auslesen der Spannung an Channel 1 des dsp3016...\n");
      device_data->getvar(&N,"dsp3016","N");
      voltage=readvoltage(N,0);
      parameters->store(voltage);
      device_data->putvar(voltage,"dsp3016","Voltage1");
    }
  return 1;
}


int dsp3016_close(List *device_data, Scheduler *parameters)
{
  printf("Closing dsp3016 ... !\n");
  if (camac_close() != 1)
    {
      return parameters->error(E_GPIB,"CAMAC ibclose err");
    }
  return 1; //function has to return a value -> 1 otherwise measurement stops
}
