//device file for the HYTEC 640 VL 16 channel 12bit DAC
//each channel has his own device file
//they all use the same initfile
//
//last edit: M. Dammasch, Apr/2001
//last edit: S.Gerlach, Apr-June/2001

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "list.hh"
#include "sched.hh"
#include "camacs.h"
#include "error.hh"

#undef DEBUG_DEV_HYTEC640

// 4V = 1638
// 5V = 2048
//#define TTL 819 

// global variables
static int  N, qx, TLAM, slowmode, chanmax, offset, max, step,ttlchannel,ttl;
static int  *shots, *enabled;

/******************* function(s) common to all channels **********************/
int hytec640_enable ( Scheduler *parameters )
{
  unsigned int  dummy;

  qx = camac(25,0,N,&dummy);
  qx=  camac(11,0,N,&dummy);

#ifdef DEBUG_DEV_HYTEC640
  printf("\nhytec640: enabled ALL channels\n",channel);
#endif

  return 1;
}

int hytec640_setvoltage ( int channel, Scheduler *parameters, unsigned int voltbits )
{
  qx = camac(16,channel,N,&voltbits);   //set output voltage  dac channel register

#ifdef DEBUG_DEV_HYTEC640
  printf("\nhytec640_%d: %d voltage set\n",channel,voltbits);
#endif

  return 1;
}

int hytec640_readout ( int channel, Scheduler *parameters )
{
  unsigned int  dummy;

  qx = camac(0,channel,N,&dummy);   //readout dac channel register
  int d = (int) dummy;

  parameters->store((parameters->act_channel() % shots[channel] == 0)?(d):(-1));  //save measured or no-meas value

#ifdef DEBUG_DEV_HYTEC640
  printf("\nhytec640_%d: %s data stored\n",channel,(parameters->act_channel() % shots[channel] == 0)?("real"):("virtual"));
#endif

  return 1;
}

int hytec640_disable ( Scheduler *parameters )
{
  unsigned int  dummy;

  qx = camac(24,0,N,&dummy);

#ifdef DEBUG_DEV_HYTEC640
  printf("\nhytec640: disabled ALL channels",channel);
#endif

  return 1;
}
/*****************************************************************************/

//init function
//called each time before meausurement starts
int hytec640_init(List *device_data, Scheduler *parameters)
{	
  unsigned int x;
  char label[12];

  if (camac_init() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't initialize device "))
      return 0;

  printf("\ninitializing hytec640\n");
  device_data->getvar(&N,"hytec640","N");
  device_data->getvar(&slowmode,"hytec640","slowmode");
  device_data->getvar(&chanmax,"hytec640","channelmax");
  device_data->getvar(&offset,"hytec640","offset");
  device_data->getvar(&max,"hytec640","max");
  device_data->getvar(&step,"hytec640","step");
  device_data->getvar(&ttlchannel,"hytec640","ttlchannel");
  device_data->getvar(&ttl,"hytec640","ttl");

  device_data->getvar(&TLAM,"all","timeout");

  shots = (int *)calloc(chanmax,sizeof(int));
  enabled = (int *)calloc(chanmax,sizeof(int));

  //set defaults
  memset(shots,1,sizeof(shots));
  memset(enabled,0,sizeof(enabled));

  for (int i = 0; i < chanmax; i++)
  {
    sprintf(label,"shots_ch%d",i);
    device_data->getvar(&shots[i],"hytec640",label);
  }

  for (int i = 0; i < chanmax; i++)
  {
    sprintf(label,"channel_%d",i);
    enabled[i] = device_data->enabled("hytec640",label);
  }

  //clear all(!) channels
  camac(25,0,N,&x);         //enable
  camac(11,0,N,&x);          //erase
  camac(24,0,N,&x);         //disable

  return 1;
}

int hytec640_pre ( List *device_data, Scheduler *parameters )
{
	for (int i = 0; i < chanmax; i++) {
		//if (enabled[i])
		hytec640_enable(parameters);

		// offset + 0 .. 100 mV = 41 steps
		// 
		// channel = 4095, integration = 1

		if (i == ttlchannel)
			hytec640_setvoltage(i,parameters,ttl*(parameters->act_channel() % 2));
		else
			hytec640_setvoltage(i,parameters,
					offset+ max*(int)fmod(step*(parameters->act_channel())-1,max)/max);




		// ramp 0 .. 100 mV (~ 41 steps)
		// channel = 41, inegration = 1 
		//hytec640_setvoltage(i,parameters,41*(parameters->act_channel()-1)/parameters->maximal_channels());

		//ramp 0 ... 10 V
		//hytec640_setvoltage(i,parameters,4095*(parameters->act_channel()-1)/parameters->maximal_channels());

		// sin-like 10V*sin()  voltage 
		//hytec640_setvoltage(i,parameters,4095*(0.5+0.5*sin(2*3.1415*(parameters->act_channel()-1)/parameters->maximal_channels())));
	}

	return 1;
}

int hytec640 ( List *device_data, Scheduler *parameters )
{
	for (int i = 0; i < chanmax; i++) {
		if ( !(hytec640_readout(i,parameters)))
			return 0;
	}
	return 1;
}

int hytec640_close(List *device_data, Scheduler *parameters)
{
	printf("\nclosing hytec640 ");
	if ( !(hytec640_disable(parameters)))
		return 0;
	
	free(shots);
	free(enabled);
	
	if (camac_close() != 1)
		if (!parameters->error(E_CAMAC,"Couldn't close device"))
			return 0;

	return 1; //function has to return a value -> 1 otherwise measurement stops
}

