/*****************************************/
/*
 *               MAX ver 2.0
 *         copyright Christian Rosen
 *                   MBI
 ******************************************/

/* 4/98 */
/*!
 *\file definitions.h
 *\brief Edit this file to configure your application.
 *
 *This file contains general definitions for all parts of
 *the programm.
 *You can change default settings and
 *configure your application for your needs in this file
 *\author C.Rosen
 */

#ifndef _DEFINITIONS_H_
#define _DEFINITIONS_H_


#define MAIN_INITFILE           "initmax\0" // Default filename of main initfile
/*! \def MAIN_INITFILE
 *Filenmae of the default resource file
 */

/*! \def DATA_PACKET_SIZE
 * Size of the network packet data space that is send via the socket.
 * Decrease it, if you only send small datasets (like scalars and short arrays, for example)
 * This will increase transmission speed
 * default : 1024
 */
#define DATA_PACKET_SIZE        128   //size of data-packets data field.
/*! \def DEFAULT_HOST
 * This host the client will connect to if no command line argument is given
 */
#define DEFAULT_HOST            "localhost" //default host to connect to
/*! \def DEFAULT_PORT
 * This port the client and the server will use for their connection. You probably won't need to change this
 *
 *several ports are used for several channels (sockets) of intercommunication
 *
 *  Port   :              Service of socket :
 *
 *DEFAULT_PORT   : DevData send to server from client
 *DEFAULT_PORT+1 : DevData send to client from server
 *DEFAULT_PORT+2 : CommandData  send to sever from client
 *DEFAULT_PORT+3 : CommandData  send to client from server
 *DEFAULT_PORT+4 : MeasuredData send from server to client
 */
#define DEFAULT_PORT            20000

/*! \def DATA_SEND_INTERVALL
 * Intervall after which buffered data is sent to the client.
 * All intervall units are milliseconds
 */
#define DATA_SEND_INTERVALL     1000  	//update of plotwidgets in millisecs.
/*! \def DATA_RECEIVED_INTERVALL
 * Period after which data received from server is dequeued
 * to be plotted and stored in the database
 * all intervall units are milliseconds
 * default : 500
 */
#define DATA_RECEIVED_INTERVALL 100     //intervall to send data to client.
/*! \def PARA_SEND_INTERVALL
 * This is the period in which the server sends channel number and dev parameter lists to client.
 * Decrease it if you want a faster update of the parameter listbox.
 * default : 500
 */
#define PARA_SEND_INTERVALL     100     //intervall to send device parameters to client.
/*! \def MAX_NAME_LENGTH
 * Maximum length of parameter and device names.
 * Increase this value, if you want to use longer names in your device parameter initfile
 */
#define MAX_NAME_LENGTH         32   // Maximum length of variable names.
/*! \def MAX_VAR_LENGTH
 * Maximum length of variable names.
 * Increase this value, if you want to use longer names in your device parameter initfile
 */
#define MAX_VAR_LENGTH          32   //Length of variable values in data packets
/*! \def  ERR_MSG_LENGTH
 * Length of individual error messages to be shown in the error widget
 */
#define ERR_MSG_LENGTH          128   	//Length of error messages. default:265
/*! \def MAX_VAR_LENGTH
 * Maximum length of device names.
 * Increase this value, if you want to use longer device names in your device parameter initfile
 */
#define MAX_DEVICE_NAME_LENGTH  32    //maximal length of a device name
#define BUFSIZE                 100000 	//only for testing. default : 65000
/*! \def CONNECT_WAIT
 * Intervall to wait for client before trying to connect next socket.
 * This is sometimes neccessary if the host, the server runs on, is very slow
 */
#define CONNECT_WAIT            1000000 	//client wait in microsecs before opening next socket.
// default : 400000
//increase this if client connects with slow polld-host
//end of network definitions

//general limits relevant for the measurement
//-> you can change things here
//channel and integrations can not exceed to bytes !!
//this is the representation in the network packets
/*! \def MAX_INTEGRATIONS */
#define MAX_INTEGRATIONS        10000 	//Maximum number of integrations per channel, default : 100000
/*! \def MAX_DEVICES
* maximum number of devices that be used*/
#define MAX_DEVICES             10   	//Maximum number of devices that can be used, default : 255

/*! \def MAX_VAR
  Maximum number of variables that can be used*/
#define MAX_VAR                 255   	//Maximum number of variables ("Channels") to store measured data in. Change this value if you need to
/*! \def ACTIVE_VARIABLES
 Maximum number of variables that can be stored per channel*/
#define ACTIVE_VARIABLES        100  	//number of var's that can be stored per channel
/*! \def MAX_ARRAY_SIZE
 * Maximal array size for arrays to store measured data in
 *	linuxlab : 10000
*/
#define MAX_ARRAY_SIZE          10000 	//size of arrays to be stored, default : 100000

//definitions for GUI look & feel -> you can change this !
#define MAIN_GUI_X_SIZE         500   	//! Default width of main window.
/*! \def MAIN_GUI_X_SIZE */
#define MAIN_GUI_Y_SIZE         544 	//! Default heigth of main window.
/*! \def MAIN_GUI_Y_SIZE */
#define MAIN_GUI_POSITION_X     100     //! X-Coordinate  of main window.
/*! \def MAIN_GUI_POSITION_X */
#define MAIN_GUI_POSITION_Y     100     //! Y-Coordinate of main window.
/*! \def MAIN_GUI_POSITION_Y */
#define MIN_BUTTON_SIZE         50    	//! Control button size. They are are quadratic.
/*! \def MIN_BUTTON_SIZE */
#define SAVE_DETECT             FALSE //TRUE    //! Automatically pop-up save Widget after measurement is finished
/*! \def SAVE_DETECT */
//end of l&f definitions

//default parameters -> You can change this
#define INT_TRIGGER_PERIOD      500   	//!default internal trigger period in ms
/*! \def INT_TRIGGER_PERIOD */
#define MONITOR_MODE            FALSE 	//! Default setting for DATAQ_MODE (FALSE = DATAQ mode)
/*! \def MONITOR_MODE */
#define UPDATE_INTERVALL        500  	//! Update intervall for plotwindow in ms.
/*! \def UPDATE_INTERVALL */
#define BUTTON_BLINK            500   	//! Blinking speed of buttons.
/*! \def BUTTON_BLINK */
#define LISTB_UPDATE_INTERVALL  2000 	//! Update intervall for device parameter listbox in ms
/*! \def LISTB_UPDATE_INTERVALL */
#define PLOT_WINDOW_UPDATE      1000 	//! Update intervall of plotwindows in ms
/*! \def PLOT_WINDOW_UPDATE */
/*!
 *Definitions for commands -> You better don't change this !
 *
 *most of commands are sent from max (client) to the polld
 *but some also vice versa
 *some are requests from max for the actual status of the measurements
 *commands defined as chars since network packets contain only unsigned chars
 */
#define START                   'a' //start measurement in polld
#define STOP                    'b' //stop     "              "
#define PAUSE                   'c' //pause    "
#define CONTINUE                'd' //cont     "              "
#define CHANNELNUMBER           'e' //
#define INTEGRATIONS            'f' //set integrations in polld
#define PRIORITY                'g' //set device and priority in polld
#define REQUEST_CHANNELNUMBER   'h' //client requests actual channel
#define REQUEST_INTEGRATIONS    'i' //client requests actual integrations
#define ERROR                   'j' //send error message to GUI for user info
#define MON_MODE                'k' //set monitor mode
#define DATAQ_MODE              'l' //set dataq mode (default)
#define RESET                   'm' //reset
#define EXT_TRIGGER             'n' //set trigger external
#define INT_TRIGGER             'o' //set trigger internal
#define SET_PERIOD              'p' //set period of internal trigger (in ms)
#define CLEAR_DEVICES           'q' //erase device parameter database
#define CLEAR_PRIORITY          'r' //erase priority database
#define KILL_YOURSELF           's' //send kill command to partner
#define SET_DAT_SEND_INTERVALL  't' //intervall to send measured data
#define DEAMON_DIES             'u' //Notify client that deamon will die
#define PRIORITY_CHECK          'v' //Tell Server to check consistency of priority database
//end of command definitions

/* *********************************
 *definitions for the plotwindow
 *you can change some things here !!!
 ****************************************/

#define P_MIN_SIZE_X            400   //!default size of plotwidget
/*! \def  P_MIN_SIZE_X */
#define P_MIN_SIZE_Y            300   //!default size of plotwidget
/*! \def  P_MIN_SIZE_Y */
#define P_LEFT_BORDER           35    //!size of plotspace in widget.
/*! \def  P_LEFT_BORDER */
#define P_UPPER_BORDER          40    //!Upper border of plotspace in plotwidget.
/*! \def  P_UPPER_BORDER */
#define P_DEF_SIZE_X            300-2*P_LEFT_BORDER
/* \def P_DEF_SIZE_X */
#define P_DEF_SIZE_Y            200-2*P_UPPER_BORDER
/* \def P_DEF_SIZE_Y */
#define Y_TICKS                 11   //!number of y-ticks in Plot widget -> change this.
/*! \def  Y_TICKS */
#define X_TICKS                 11   //!number of x-ticks in PlotWidget -> change this.
/*! \def  X_TICKS */
#define TICKSIZE                5    //!size of ticks  -> change this.
/*! \def TICKSIZE */
#define TICKPOS                 30   //!distance of ticks to axis.
/*! \def TICKPOS */
#define TICKPOS_X               15   //!distance of ticks to  x-axis.
/*! \def TICKPOS_X */
#define HGRIDLINES              TRUE; //!ENABLE/DISBALE horzontal gridlines.
/*! \def HGRIDLINES */
#define VGRIDLINES              FALSE; //!ENABLE/DISABLE vertical gridlines.
/*! \def VGRIDLINES */
#define Y_UPPER_LIMIT           2      //!default y_limits for plotting range.
/*! \def Y_UPPER_LIMIT */
#define Y_LOWER_LIMIT           -5.000 //!Lower limit of y-axis, the user can change this
/*! \def Y_LOWER_LIMIT */
#define AUTOSCALE               TRUE //!Default setting of autoscaling.
/*! \def AUTOSCALE */
#define AS_INTERVAL             1    //!Points to exceed limit before autoscaling starts.
/*! \def AS_INTERVAL
 * This variable contains the number of points that have to
 * exceed the upper/lower plot limit, before autoscaling starts.
 * By setting this variable to a value > 1 you can avoid the Plot Widget to rescale for each single overshoot. Only works if austoscale is enabled
 */
#define MON_MODE_CHANNELS       1000 //!width of plotwindow in monitor mode, also used for main routine
/*! \def MON_MODE_CHANNELS
 * This value basically gives the x-size of the plot Windopw in monitor mode. If MON_MODE_CHANNELS is reached the window is erased and replotted.
 */
//definitions for styles
#define SOLIDLINE               1    //Linestyle for Plotwindow

#define POINTS                  2    //Draw points or lines in plotwindow

#define MIN_ARRAY_LENGTH        10   //!If the array is smaller than this value, the window will be expanded to this size.
/*! \def MIN_ARRAY_LENGTH
 * This is basically the minimum size of the Plotwindow for plotting arays.
 * This is neccessary because a window width of for example 10 looks pretty ugly..
 */
#define AXISCOLOR               red    	//!Color of axis in Plot Window
/*! \def AXISCOLOR color settings for axis, graph and background of the plot
 *
 * In Qt, predefined colors are (You have to use these):
 * black, white, darkGray,
 * gray, lightGray, red, green, blue, cyan, magenta, yellow,
 * darkRed, darkGreen, darkBlue, darkCyan, darkMagenta,
 * darkYellow, color0 and color1.
 * the user can set these colors here or in the initfile
 * use the predefined QColor defintions !
 */
#define LINECOLOR               white   //! Default color for plotting scalars
/*! \def LINECOLOR
 *
 * In Qt, predefined colors are (You have to use these):
 * black, white, darkGray,
 * gray, lightGray, red, green, blue, cyan, magenta, yellow,
 * darkRed, darkGreen, darkBlue, darkCyan, darkMagenta,
 * darkYellow, color0 and color1.
 * the user can set these colors here or in the initfile
 * use the predefined QColor defintions !
 */
#define ARRAYLINECOLOR          yellow  //! Default color for plotting arrays
/*! \def ARRAYLINECOLOR
 *
 * In Qt, predefined colors are (You have to use these):
 * black, white, darkGray,
 * gray, lightGray, red, green, blue, cyan, magenta, yellow,
 * darkRed, darkGreen, darkBlue, darkCyan, darkMagenta,
 * darkYellow, color0 and color1.
 * the user can set these colors here or in the initfile
 * use the predefined QColor defintions !
 */
#define LABELCOLOR              white   //! Color of axis ticks and Labels in the Plot Window
/*! \def LABELCOLOR
 *
 * In Qt, predefined colors are (You have to use these):
 * black, white, darkGray,
 * gray, lightGray, red, green, blue, cyan, magenta, yellow,
 * darkRed, darkGreen, darkBlue, darkCyan, darkMagenta,
 * darkYellow, color0 and color1.
 * the user can set these colors here or in the initfile
 * use the predefined QColor defintions !
 */
#define BACKGROUND              black  //!Background color of Plot Window.
/*! \def BACKGROUND
 *
 * In Qt, predefined colors are (You have to use these):
 * black, white, darkGray,
 * gray, lightGray, red, green, blue, cyan, magenta, yellow,
 * darkRed, darkGreen, darkBlue, darkCyan, darkMagenta,
 * darkYellow, color0 and color1.
 * the user can set these colors here or in the initfile
 * use the predefined QColor defintions !
 */
/****************************************/
/* end plotwindow definitions           */
/****************************************/

/****************************************/
/*definitions for the save Widget      */
/****************************************/
//change this size as you like
//
#define SAVE_WIDGET_X_SIZE      440 //! Default width of Save Widget.
/*! \def SAVE_WIDGET_X_SIZE */
#define SAVE_WIDGET_Y_SIZE      330 //! Default heigth of Save Widget.
/*! \def SAVE_WIDGET_Y_SIZE */

#endif
