/**************************************************************************
 * $Id: devices.h,v 1.1 1998/10/17 19:07:27 rosen Exp rosen $
 *
 * Revision 1.01 1998/10/03 14:34:15  rosen
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **************************************************************************/

 /*! \file devices.h 
  *\brief Header file for all Device_functions : 
  * Here you find informations on how to implement new device functions into the program.
  * 
 *This is the header file for all function declarations.
 *This file contains the device function pointer list
 *Here you have to add the name and the declaration 
 *of the device_function of your new device.
 *\b Don't \b forget \b to \b set \b the \c NUMBER_DEVICES \b variable to number of devices in the device_vectors.
*/

#ifndef DEVICES_H
#define DEVICES_H
#include "list.hh"   //this is class that contains the device parameter database
#include "sched.hh"  //this class does the datastorage before it is send to client 
 //
 //enter the device-function declaration here !!!!!!!
 //each device function should have it's declaration here
 //
 /* This is an example device module 
  * It generates artificial data, to simulate a measurement process 
  * This is the \b init -function. It is called before hte measurement starts
  */
int adc_init   (List *, Scheduler *);
/* \fn int adc_init(List *,Scheduler *) 
 *This function is the \b init \b function of the \b device-module adc.
*It is called before once before the measurement starts. It is used to reset and/or initialize the devices in your experiment.
* \param *device_data a pointer to the list class. This class contains the database access functions
* \param *parameters a pointer to the Scheduler class. This class contains the functions to store the measured data. For example  : void Scheduler::store(int ) 
*/
int adc        (List *, Scheduler *);
/* \fn int adc(List *,Scheduler *) 
 *This function is the \b main \b function of the \b device-module adc.
* Here the data is taken and stored into the databasse by calling the function void Scheduler::store (int )
* \param *device_data a pointer to the list class. This class contains the database access functions
* \param *parameters a pointer to the Scheduler class. This class contains the functions to store the measured data. For example  : void Scheduler::store(int ) 
*/
int adc_pre    (List *, Scheduler *);
/* \fn int ac_pre((List *, Scheduler *) 
 *This is the \b pretrigger function of the device module \b adc.
* This function is called before the main function int adc(List *, Scheduler *) is called.
* These functions are used to reset the device before the integrations cycle starts.
*/

adc2249_init (List *, Scheduler *);
adc2249_pre (List *, Scheduler *);
adc2249 (List *, Scheduler *);
adc2249_close (List *, Scheduler *);

/****************************************
 *this variable contains the number of
 *devices in the Function_List vector
 *MAKE SHURE IT IS COORECT !!!!!
 *****************************************/

#define NUMBER_DEVICES 1
/*! \def NUMBER_DEVICES 
* The \cNUMBER_DEVICES definition contains the  number of devices in the device Vector.
* You \b have \b to \b set \b this \b value \b!\b!
* To the number of elements int that vector !
*/

/**********************************************************************
 * 
 *List of device function pointers 
 *
 *pointers are used by the polld to calls the device routines
 *they MUST be in the same order than in the priority file !!!!!!
 *otherwise the wrong device is called
 *Don't forget to set NUMBER_DEVICES correctly
 **********************************************************************/
/*! List of pointers to the device functions.
 * The Scheduler uses this list to call the device functions. 
 * You have to insert the \e function-names of your device functions here.
 * Each functions is then defined by a number, it's \e priority.
 * This means, that the order of the devices in this vector has to \e be \e in \e the \e same \e order than  
 * int the \e priority-file.
 * There are four Arrays of vectors ! Don't forget any of them.
 * In each vector the order of the devices has to be same.
 */
int (*Function_List      [NUMBER_DEVICES]) (List *, Scheduler *)  = 
{
adc2249
};
/*! List of pointers to the \b Pretrigger device functions.
 * The Scheduler uses this list to call the device functions. 
 * You have to insert the \e names of your device functions here.
 * Each functions is then defined by a number, it's \e priority.
 * This means, that the order of the devices in this vector has to \e be \e in \e the \e same \e order than  
 * int the \e priority-file.
 * There are four Arrays of vectors ! Don't forget any of them.
 * In each vector the order of the devices has to be same.
 */
int (*Pre_Function_List  [NUMBER_DEVICES]) (List *, Scheduler *)  = 
{
adc2249_pre
};
/*! List of pointers to the \b Init device functions.
 * The Scheduler uses this list to call the device functions. 
 * You have to insert the \e function-names of your device functions here.
 * Each functions is then defined by a number, it's \e priority.
 * This means, that the order of the devices in this vector has to \e be \e in \e the \e same \e order than  
 * int the \e priority-file.
 * There are four Arrays of vectors ! Don't forget any of them.
 * In each vector the order of the devices has to be same.
 */
int (*Init_Function_List [NUMBER_DEVICES]) (List *, Scheduler *)  = 
{
adc2249_init
};
/*! List of pointers to the \b Close device functions.
 * The Scheduler uses this list to call the device functions. 
 * You have to insert the \e function-names of your device functions here.
 * Each functions is then defined by a number, it's \e priority.
 * This means, that the order of the devices in this vector has to \e be \e in \e the \e same \e order than  
 * int the \e priority-file.
 * There are four Arrays of vectors ! Don't forget any of them.
 * In each vector the order of the devices has to be same.
 */
int (*Close_Function_List[NUMBER_DEVICES]) (List *, Scheduler *)  = 
{
adc2249_close
};


#endif

//DONT'T WRITE ANYTHING BEHIND THE #endif LINE !!!!!

