/**
   GPIB Interface for IEEE488/GPIB devices controlled by the MAX program

   (c) 1998 by C. Rosen, C. Otto, D. Rohloff, M. Dammasch

   last edit: 1998/08/13 by M. Dammasch
**/ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <ib.h> 
#include "gpiberr.h"
#include "gpib.h"


GPIB_Device  gpib_init ( char *device_name, int initflag )
{

  GPIB_Device  newdevice;

  strcpy(newdevice.name,device_name);

  printf("\n***** Connecting GPIB Device '%s' *****",newdevice.name);

  if((newdevice.addr = ibfind(newdevice.name)) & ERR)
  {
    gpib_error("ibfind error for '%s'@%02d",newdevice.name,newdevice.addr);
    newdevice.stat = GPIB_ERR;
    return newdevice;
  }

  printf("\nGPIB-Address for '%s' is set to #%d",newdevice.name,newdevice.addr);
    
  if (initflag & IB_IFC)
  {
    printf("\nsend IFC"); //ibsic aus Lib. 
    if( ibsic(newdevice.addr) & ERR)
    {
      gpib_error("ibsic error for '%s'@%02d",newdevice.name,newdevice.addr);
      newdevice.stat = GPIB_ERR;
      return newdevice;
    }
    printf(" done ");
  }

  if (initflag & IB_REM)
  {
    printf("\nsend REM");
    if( ibsre(newdevice.addr,1) & ERR)
    {
      gpib_error("ibsre error for '%s'@%02d",newdevice.name,newdevice.addr);
      newdevice.stat = GPIB_ERR;
      return newdevice;
    }
    printf(" done ");  
  }

  if (initflag & IB_CLEAR)
  {
    printf("\nclearing GPIB device '%s' ...",newdevice.name);
    
    if( ibclr(newdevice.addr) & ERR)
    {
      gpib_error("clear error for '%s'@%02d ",newdevice.name,newdevice.addr);
      newdevice.stat = GPIB_ERR;
      return newdevice;
    }

    printf(" done ");
  }

  printf("\n***** Completed Successfully *****\n");

  newdevice.stat = GPIB_OK;
  return newdevice;
}

int  gpib_clear ( GPIB_Device *device )
{
  printf("\nclearing GPIB device '%s' ...",device->name);

  if( ibclr(device->addr) & ERR)
  {
    gpib_error("clear error for '%s'@%02d ",device->name,device->addr);
    device->stat = GPIB_ERR;
    return device->stat;
  }

  printf(" done ");
  device->stat = GPIB_OK;
  return device->stat;
}

int  gpib_write ( GPIB_Device *device, char *formatstr, ... )
{
  char     cmdstr[GPIB_STRMAX+1];
  va_list  args;

  va_start(args,formatstr);
  vsprintf(cmdstr,formatstr,args);
  va_end(args);

#ifdef DEBUG
  ibsta = iberr = ibcnt = 0;
#endif

  if ( ibwrt(device->addr,cmdstr,strlen(cmdstr)) & ERR)
  {
#ifdef DEBUG
    gpib_error("write error for '%s'@%02d due to\ncouldn't send <%s> ",device->name,device->addr,cmdstr);
#else
    gpib_error("write error for '%s'@%02d ",device->name,device->addr);
#endif    
    device->stat = GPIB_ERR;
    return device->stat;
  }

#ifdef DEBUG
  printf("\nGPIB_Write['%s'@%02d](%d/%d):<%s> {Sta: %d|Err: %d|Cnt: %d}",device->name,device->addr,ibcnt,strlen(cmdstr),cmdstr,ibsta,iberr,ibcnt);
#endif

  device->stat = GPIB_OK;
  return device->stat;
}

int  gpib_read ( GPIB_Device *device, char *buffer, unsigned long length )
{
  ibcnt = 0;

#ifdef DEBUG
  ibsta = iberr = 0;
#endif
 
  if ( ibrd(device->addr,buffer,length) & ERR)
  {
#ifdef DEBUG
    gpib_error("read error for '%s'@%02d due to\nreceived <%s> ",device->name,device->addr,buffer);
#else
    gpib_error("read error for '%s'@%02d ",device->name,device->addr);
#endif    
    device->stat = GPIB_ERR;
    return device->stat;
  }

  if (length > (unsigned int)ibcnt) buffer[ibcnt] = '\0';

#ifdef DEBUG
  printf("\nGPIB_Read['%s'@%02d](%d):<%s> {Sta: %d|Err: %d|Cnt: %d}",device->name,device->addr,ibcnt,buffer,ibsta,iberr,ibcnt);
#endif

  device->stat = GPIB_OK;
  return device->stat;
}

















