/**************************************************************************
 * $Id: list_client.hpp,v 1.1 1998/10/17 19:01:31 rosen Exp rosen $
 *
 * Revision 1.01 1998/10/03 14:34:15  rosen
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **************************************************************************/
//list header file contains class declaration for list class
//the list class stores the device parameters which can be seen and edited
//in the listbox
//it also provides all functions to access it

#ifndef LIST_CLIENT_H
#define LIST_CLIENT_H

#include "packet.h"

#include <iostream.h>
#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//STL includes
#include <string>

#include <qstring.h>
#include <qlist.h>
/*
 *Plot Properties list element class PlotProps :
 *Max stores the properties of all plot statements in an additional list
 *to initialize the plotwindows with it
 *determine the properties of the plotwindows for all variables
 *to be plotted, if data is not given, set default values from
 *definitions.h
 *
 *set all options to default values from the file definitions.h
 *these are all plot properties the user can set in the initfile
 *the new plotwindows are then initialized with this data
 *see also definitions.h and plot.h
 */
class PlotProps {
public :
  friend class Plot;
  PlotProps (char *dev = "no_name",char *var="no_name",
	     int in =0,int prio=0,bool p=FALSE,bool s=FALSE) {
    dev_name = dev;
    var_name = var;
    priority  = prio;
    indx      = in;	  //number of var with plotlag == TRUE in list
    plotflag  = p;	  //if plotflag is == FALSE -> discard the data and don't plot it
    storeflag = s;	  //if storeflag == FALSE then  index = -1
    InitPlotOptions();
    marker = FALSE;
  }
  ~PlotProps() {		//default destruktor
    dev_name.~string();
    var_name.~string();
  }
  int    indx;         //this index is equal to the variable index received from polling deamon
  int priority;
  string dev_name;
  string var_name;
  bool   plotflag;   //really plot it or discard queued potdata ?
  bool   storeflag;  //is it present in the database of measured data?
  PlotSettings Ps;   //this structure holds all Plotoptions -> see : packet.h
  bool   marker;     //used for finding the right plotwindow
private: 	     //these are priority of this device
  void InitPlotOptions(void) {
    Ps.pminx        = P_MIN_SIZE_X;
    Ps.pminy        = P_MIN_SIZE_Y;
    Ps.pleftborder  = P_LEFT_BORDER;
    Ps.pupperborder = P_UPPER_BORDER;
    Ps.pdefsizex    = P_DEF_SIZE_X;
    Ps.pdefsizey    = P_DEF_SIZE_Y;
    Ps.xticks       = X_TICKS;
    Ps.yticks       = Y_TICKS;
    Ps.ticksize     = TICKSIZE;
    Ps.tickpos      = TICKPOS;
    Ps.tickposx     = TICKPOS_X;
    Ps.as_interval  = AS_INTERVAL;
    Ps.yupper       = Y_UPPER_LIMIT;
    Ps.ylower       = Y_LOWER_LIMIT;
    Ps.autoscale    = AUTOSCALE;
    Ps.hgridlines   = HGRIDLINES;
    Ps.vgridlines   = VGRIDLINES;
  }
};


//the data_type flag contains the variable type of the save variable
// -> see : Cl_List::get_data_type(char *)
//this is neccessary to determine the type for saving the data
//(overloading doesn't work here) -> But, I don't need this routine anymore
//Scheduler::store(var) now sets up an array during first measurement cycle to store data types
//look there to figure that out
//IMPROVEMENT :
//BUT ADD TEMPLATE LIST CLASS TO REMOVE SWITCH STATEMENTS IN GETSTRING AND
// GETALLDEVICEPARAMETERS functions
//
//the following class contains one element of the linked list of device parameters
class Element {
 friend class Cl_List;
public:

private:
//private Konstructor for int var
  Element(int W, const char *device, const char *var,int flag,int data_type,
	int x=0,int y=0, Element *E = 0) {
    i_var       = W;
    next        = E;
    enabled   = flag; //this is the enabled flag !!!! TRUE -> device stores the data
    plotflag    = y;   //TRUE is var is plotted
    type        = data_type;
    storeflag   = x;   //TRUE if variable is a store variable
    device_name = device;
    var_name    = var;
  }
//private Konstructor for double var
 Element(double W, const char *device,const char *var,int flag, int data_type,
	int x=0,int y=0, Element *E = 0) {
    d_var       = W;
    next        = E;
    enabled     = flag; //this is the enabled flag
    plotflag    = y;    //plot ot ?
    type        = data_type;
    storeflag   = x;  //indicates that thiselemet is a "store" variable
    device_name = device;
    var_name    = var;
 }	
  ~Element() {
    device_name.~string();
    var_name.~string();
  }
  string   device_name;
  string   var_name;     //data ist a reserved word for a variable name
  int      enabled;    //if that is nonzero the according variable
  //is saved to disk and can be plotted
  //1 = int,2 = double, 3 =char, 4 = string
  int      i_var;
  double   d_var;
  char     c_var;
  char *s_var;
  Element  *next;
  int       type;      //type flag 1 = int,2 = double, 3 = char, 4 = string
  int       storeflag; //TRUE is var is a store variable
  int       plotflag;  //TRUE is variable is to be plotted
};
//
//This class represents the list of device data together with the functions to
//access it
class Cl_List
{
public:
  Cl_List()
  {
    L=0; Buffer_pointer=L;
    PlotPropList = new QList<PlotProps>;
    PlotPropList -> setAutoDelete(TRUE);
    assert(PlotPropList !=0);
    //   ParameterStr = new char[10000];
    //assert(ParameterStr !=0);
  }
  ~Cl_List() {
    remove();
    delete PlotPropList;
  }
  void initialize();
  void insert  (int   , char *,char *,int,int ); //insert new data-element
  void insert  (double, char *,char *,int,int ); //dito but with double
  void append  (int,    char *,char *,int,int,int,int ); //append new data-element (at the end)
  void append  (double ,char *,char *,int,int,int,int ); //dito but with double-variable
  Element * exist(char *,char *);                //check for deviceName && varName in the list
  void SortPlotProperties(void);          //add the actual priorites to the plot properties list
  void remove  ();                        //removes complete list from memory
  int is_active(char *);                  //returns 1 is char* device_name is an active device,
                                          //otherwise -1
  int    remove (int );                   //removes specific element from list
  int    remove (double );
  int    getvar (int *   ,char *,char *); //first  device then var-name
  int    getvar (double *,char *,char *); //same for double vars
  int    putvar (int     ,char *,char *); //set variable value (int)
  int    putvar (double  ,char *,char *); //set variable value (double)
  int    varcomp(Element *,char *,char *);
  void   putvar(int, int);                //set new variable for element at position i
  void   putvar(double, int);
  void   putvar( char *,int);
  int    enabled(char *,char *);          //return TRUE if this var is enabled
  int    plotstatus(int );                //return plotstaus(nostorevar,enabled,plot)
  int    getint   (int );                 //return variable of element at position index
  double getdouble(int );
  char   getchr   (int );
  char*  getstr   (int );
  char*  get_data_type(char *);           /* returns a string with a list of the
	 *saved data type of device char
	 *f.e. : the variables string,double,int
	 *would return : sdi   *
	 * needed for the save-routine*/
  /*network related functions only in ver 2.0 !!!*/
  devDataPacket * GetPacketElement(char *); //return struct packet for socket transfer
  devDataPacket * GetNextPacket(void );   //return data element pointed to by BufferPointer
  void   ResetHelpPointer(void);         //reset help list pointer to beginning of list
  int    AdvanceHelpPointer(void);       //go to next element
  /* end */
  string  getAllDeviceParameters(char * ); //returns string containing all data of dev. QString name
  void   reset_data_buffer();             //which is needed for parameter file save routine
  void   printlist();
  int    initlist(char *);                //setup database from initfile, return -1 on error
  int    initPlotOptions(FILE *fp,PlotProps *); //returns enabled flag
  Element* End_Of_List();
  char *  getstring(int);                  //returns string with dev-name par-name value
  int    get_type(int );	      	  //returns type of ith element		
  int    length();                        //Number of scalarelements in the list
  int    array_length();	          //Number of array elements in the list
  char * get_name(int );                  //return name of device at position i
  const char * get_var_name(int );              //return var name of device at position i
  int    get_store_flag(int );            //return the storeflag out of parameter database
  int    get_plot_flag(int);      //check if variable specified by position contains plotdata (ysize)
  int    get_enabled_flag(int);
  void   set_enabled_flag(int,bool);        //toggle the flags according to user input
  void   set_plot_flag(int,bool);         //"switch" on/off a variable
  int    get_number_plotvars(char *);     //return number of variables to be plotted
  string get_plotvar_namelist(char *);   //defined by "data" statement in initfile
                                          //or zero if list is empty or at the end
  QList <PlotProps> *PlotPropList;
private:
  string ParameterStr;
  int index;    //index for plotvariable
  int x,i,y;
  Element *L;
  Element *BufferPointer,*Buffer_pointer;
};

#endif

//Don't write behind the endif line !!

