/**************************************************************************
 * $Id: max.hpp,v 1.8 1999/02/21 18:47:43 rosen Exp rosen $
 *
 * Revision 1.18  1998/10/03 14:34:15  rosen
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **************************************************************************/
/* header file
 * main -client- programm with Qt-GUI
 */
#ifndef _MAX_H_
#define _MAX_H_

#include "definitions.h"
#include "network.h"
#include "packet.h"
#include "qtincludes.h"
#include "list_client.h"
#include "plot.h"
#include "priority.h"   //priority list
#include "utils.h"
#include "database.h"   //the main database for measured data
#include "error.hh"

//STL includes
#include <vector.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>

//!
//!this class ist used in a list to buffer the measured data
//!before it is plotted
//!
class PlotBuffer  {
public :
  PlotBuffer(int    value=0,int channel=0,int index=0,
	     int priority=0);
  PlotBuffer(double value=0, int channel=0,int index=0,
	     int priority=0);
  PlotBuffer(int    *a=0,int len=0, int channel=0,int index=0,
	     int priority=0);
  PlotBuffer(double *a=0,int len=0, int channel=0,int index=0,
	     int priority=0);
  PlotBuffer(vector<int> *vptr=NULL,int len=0,int channel=0,int index = 0, int priority = 0); //buffers plotdata
   PlotBuffer(vector<double> *vptr=NULL,int len=0,int channel=0,int index = 0, int priority = 0); //buffers plotdata
  ~PlotBuffer() {
    if (i_array) delete i_array;
    if (d_array) delete d_array;
  }
  int    ival()  const    { return i_val;  }
  double dval()  const    { return d_val;  }
  int *  iarr()  const    { return i_array;}
  double*darr()  const    { return d_array;}
  vector<int>    *ivec() const { return iVecPtr; }
  vector<double> *dvec() const { return dVecPtr; }
  int    ind()   const    { return indx;   }
  int    prior() const    { return prio;   }
  int    chann() const    { return chan;   }
  int    len()   const    { return length; }
  void   setindex(int x)  { indx = x;}
  int    var_type() const { return type; }
private :
  int     i_val;
  double  d_val;
  int    *i_array;
  double *d_array;
  int     length;
  int     type;
  int     chan;
  int     prio;
  int     indx;
  int     x;
  vector<int>    *iVecPtr;    //Pointers to Vectors in database
  vector<double> *dVecPtr;
};
class MainGUI : public QMainWindow //public QWidget
{
Q_OBJECT;
 public:
  MainGUI(char *hostname,char *filename);
  ~MainGUI();
  int  readMainInitfile(char *);
 protected:
  void signalhandler( int );
  int devDataConnect  ( char *, int ); //establish devData sockets
  int cmdSocketConnect( char *, int ); //establish command socket
  int datSocketConnect( char *, int ); //establish data socket connection
  int connectSockets(char *);  //Connect all sockets to char * hostname
  void sendDeviceData(unsigned char *,unsigned char *,
		      unsigned char *,unsigned char ,unsigned char);
  int  sendDeviceData(devDataPacket *);
  void sendAllDeviceData(void );
  void sendAllPriorities(void);   //send priority database to polld
  //send command line + additional info to the polld
  int  sendCommand   (char ,char *,int);
  signals:
  void stop (int  );
  void timeout( );
  void timeout2( );
  void timeout3(); //timeout to update channel number widget
  protected slots :
  //virtual void dataReceived(int );
  // virtual void timerEvent(QTimerEvent *e);
  void commandReceived();
  void dataReceived();
  void parameterReceived();
 public slots :
  void open ();
  void save ();
  void do_save(const char *,const char *,
	       const char *,int ,char * ,
	       const char *,const char *);
  void about();
  void print();
  void help ();
  void bye  ();
  void browsePrio();
  void browseDev();
  void browseMain();
  void reinit();
  int  reinitAll();
  void reinit_priority();
  void reinit_devices();
  void startclicked();
  void stopclicked();
  void contclicked();
  void resetclicked();
  void pauseclicked();
  void toggle_trigger_mode();
  void toggle_monitor_mode();
  void listbox_update();
  void priority_update();
  void set_plotwindow_update_intervall();
  void set_trigger_internal();
  void set_trigger_external();
  void set_trigger_delay();
  void toggle_save_widget_enable();
  void toggle_save_widget_disable();
  void apply_trigger_delay(uint ); //send new delay to polld
  void apply_update_intervall(uint, uint );
  void apply_options(int,int);      //called from options widgets
  void select_plotvars();
  void set_maxnum();
  void set_integrations();
  void set_new_parameter();  //these two slots deal with dev_parameter editor
  void edit_parameter(int ); //called when user double clicks a device lbItem
  void edit_priority(QListViewItem *);   //same for priorites listbox (the right one)
  void prio_lv_PopUp( QListViewItem *,const QPoint &, int);
  void set_priority();
  void Prio_Enabled();
  void Prio_Disabled();
  void edit_pfile();        //signals for edit of initfile filenames
  void edit_dfile();        //devices
  void edit_ifile();        //main initfile
  //
  //timer events....
  void ButtonTimerEvent();  //timer to make pause button blink
  void RefreshNumber();     //channelnumber widget updated periodically
  void UpdatePlotWindows(); //empty Plotbuffer and send to p_window
  void SetWindowParameters(Plot *,PlotProps *);
  void appendData(dataPacket);
private:
 /* max' own classes used for data&parameter handling
   *and some widgets... */
  void SortPlotProperties();   //bring list in right order after priorites have changed

  Database     *Schedule;
  SaveWidget   *SaveW;
  UpdateWidget *W;
  DelayWidget  *DelayW;
  Cl_List      *Devices;       //data base of device data
  Pr_list Priority;            //data base for device priorites
  /*
   *  network stuff
   */
  QWidget *         Main_Widget;
  QSocketNotifier  *RDataNotifier; //Not. to receive device data from polld
  QSocketNotifier  *RCmdNotifier;  //Notifier to receive commands from polld
  QSocketNotifier  *DataNotifier;  //Notifer about measured data socket
  int              data_sfd;      //socket-filedescriptor for parameter socket
  int              mdat_sfd;       //sfd for measured data
  int              cmd_sfd;       // "         "   for Receivecommand sockets
  int              cmdr_sfd;      //command receive socket filedescriptor
  int              devr_sfd;      //socket     "      for Receiving DevData
  int 		   devs_sfd;      //sockfd to send par data
  int              timerID;
  int              port;          //port for sockets
  int              flag;          //used in ioctl comd
  int              pid;           //process ID of child
  char *           hostname;      //host to connect to
  struct hostent   *host,*hostBackup;
  struct hostent   *hp;
  struct sockaddr_in server,server_buffer; //pold is server
  struct sockaddr_in client;              //max is client
  struct sockaddr_in client_polld;         //polld is client sends devdata to max
  struct sockaddr_in server_max;           //max is server, seceives devData from polld
  struct sockaddr_in cmdClient_max;        //client/server address for the two
  struct sockaddr_in cmdClient_polld;      //command exchange sockets
  struct sockaddr_in cmdServer_max;
  struct sockaddr_in cmdServer_polld;
  struct sockaddr_in dataClient;           //max is client to receive data
  struct sockaddr_in dataServer;           //address for polld
  struct sockaddr_in server_polld;
  unsigned int     server_len;
  int              dataClient_len;
  devDataPacket    d_packet;  //Packet for sending dev_parameters
  command_packet   c_packet;  //Packet for sending commands and measurement infos
  dataPacket       packet;    //Packet containing measured data
  int              n;
  int              cmdClient_max_len;
  int              cmdvalue; //value of received command
  /*
   *end network stuff
   */

  /*
   *file-parameters for max
   */
  char       *pr_filename;    //  [80];  //filename of priority list initfile
  char       *init_filename;  //  [80];  //filename of device_data initfile
  char       *max_init_filename;//[70];  //filename of general initfile
  char       save_fname       [100]; //filename of measured data saved to disk
  char       array_fname      [100]; //filename of measured int array file
  char       image_fname      [100];
  char       data_dir         [100]; //path of directory to store data in
  char       *l_fname;
  char       *l_afname;
  char       *l_ifname ;
  char       *f_name;
  /*
   *QT-Widgets
   */
  bool        eventFilter(QObject *,QEvent *);
  QMenuBar    *menu;
  QToolBar    *fileTools;
  QLabel      *label;
  QLabel      *channel;
  QLabel      *label_max_channels;
  QLabel      *integrations;
  QLabel      *label_integrations;
  QLabel      *label_measurement;
  QLabel      *prio_label;
  QLabel      *dev_label;
  QLabel      *init_label;
  QLCDNumber  *chan_no;
  QPopupMenu  *options;
  QPopupMenu  *priority_trigger;
  QFrame      *top_separator;
  QFrame      *bottom_separator;
  QPushButton *button_triggermode;
  QPushButton *button_monitormode;
  QPushButton *button_speed;
  QPushButton *pbcont;
  QPushButton *pbstart;
  QPushButton *pbstop;
  QPushButton *pbreset;
  QPushButton *pbpause;
  QProgressBar *progress;
  QProgressBar *int_progress;
  QLabel      *progressTitle;
  QCheckBox   *cbutton_update;
  QLabel      *chn_no;
  QLabel      *chn_label;
  QListBox    *device_lb;
  QListBox    *prio_lb;
  QListView   *prio_lv;        //ListView for Priority listbox
  QLineEdit   *dev_editor;
  QLineEdit   *integ_editor;
  QLineEdit   *channel_editor;
  QLineEdit   *prio_editor;
  QLineEdit   *prio_fname;
  QLineEdit   *dev_fname;
  QLineEdit   *in_fname;
  QCursor     *mouse;
  QTimer       *blink_timer;   //pause button blinking
  QTimer       *number_timer;  //timer for update of channel number
  QTimer       *update_timer;   //timer to empty plot_data queue
  QColor       col;            //color object for stopbutton
  QColor       col_off,col_on; //blink colors for pause button
  QColor       editcol;        //color for activated line edit boxes
  QColorGroup  colgrp;
  QColorGroup  colgrp_off,colgrp_on;
  QColorGroup  grp_active,grp_passive;


  // All the rest ...
  bool         dev_lb_update; //flag for update of devpar listbox
  bool         save_flag;   //is data saved
  bool         device_lb_flag;
  int          *lbtodata;  //array maps dev listbox index to database index
  int          *lbtoplotprops; //array to map listboxpostion to PlotProperties Listposition
  int          m_running;     //TRUE if measurement is runnning
  int          trigger_period;//period of internal trigger
  int          monitor_mode;  //TRUE = dataq mode, FALSE = mon_mode
  int          trigger_mode;  //TRUE = external, FALSE = internal
  int          update_intervall; //timer intervalls to update widgets
  int          plot_update;
  int          data_send_intervall; //timer value of Polld' sendData timer
  int          color_state;
  int          pause_flag;
  int          stop_flag;
  int          continue_flag; //to enable continue button
  int          max_channels;
  int          max_integrations;
  QDate        date;
  QTime        time;                  //actual time of measurement
  QString      namestr;
  Plot         *Plotwin;
  QList<Plot>  *WindowList;      //this list contains the pointers to plotwins
  QList<PlotBuffer> *P_Buf;  	 //queue to buffer plotdata
  vector<int> WindowPtr;         //this vector helps to identify each variable with it's plotwindow
  //vector[var_index] = index of element in WindowList.
  //this is the Plotwindow for variable with index var_index
  //one plot window for each variable
  //
  //variables for handling of measured data and it's storage
  //
  VarType     *VarListPtr;      //order to save data in
  ArrayType   *ArrayListPtr;    //order to save arrays in
  int         *iarraybuffer;    //buffer to stack packets together
  double      *darraybuffer;
  int          bufpos;  //# of elements in the arraybuffer
  int         *buf;             //buffer to copy data from packets into
  double      *dbuf;
  int          varcounter;      //number of vars per channel
  int          arraycounter;    //number of arrays per channels
  int          dev_index;       //index of dataset to modifiy plot flag in
  int          plot_index;      //same index in PlotProperties list
  int          windowcounter;
  bool	       m_enable;       //enable measurement
  bool	       save_detect_enable; //enable automatic popup of save-widget
};

#endif



