/*****************************************/
/*               MAX ver 2.0
 *
 ******************************************/

/* 4/98 */
/*
 *packets.h
 *declaration for socket datagramm packets
 *
 */
/* chain _byte :
 * bit 0 : first packet of sequence
 * bit 1 : middle packet
 * bit 2 : last packet of sequence
 */

#ifndef _PACKET_H_
#define _PACKET_H_
#include "definitions.h"
#pragma pack(1)     /*byte alignment*/
typedef struct {                       //starts with header
  unsigned char chain_byte;            //for this data set
  unsigned char type;                  //a = int,b=double,c = char;
  unsigned char device_index[sizeof(int)];
  unsigned char priority    [sizeof(int)];    //device it belongs to
  unsigned char var_index   [sizeof(int)];  //channel of this device
  unsigned char t_length    [sizeof(int)];    //length of total data sequence
  unsigned char data_length [sizeof(int)]; //length of this buffer
  unsigned char channel     [sizeof(int)];		
  unsigned char buffer      [DATA_PACKET_SIZE]; //measured data
} dataPacket;
#pragma pack()
#define FULL_P_SIZE sizeof(dataPacket) //total size of packet data+header

/*command packet :
 * contains commands for client GUI to control the polld :
 * for exact command description see definitions.h
 * channel numbers and priorities can not exceed 16bit (65536)
 */
#pragma pack(1)                         //byte alignment
typedef struct {
  unsigned char command;                //command string
  unsigned char name[MAX_NAME_LENGTH];  //buffer for device name
  unsigned char data[sizeof(int)];      //databyte	
  unsigned char data_high;              //high data byte
  unsigned char error_msg[ERR_MSG_LENGTH]; //user error message
} command_packet;
#pragma pack()

#define CPACKETSIZE sizeof(command_packet)

/* packet to transmit device data information
 * type :
 * 0 = dataset for device data
 * >0  = error message
 * var name contains user defined message
 * length is limited to 255 bytes
 */
#pragma pack(1) /*byte alignment*/
typedef struct {
  unsigned char length;
  unsigned char type;      // 'i'= integer, 'd' = double, 's'= string, 'c' = char
  unsigned char status;    // 'a' enabled, 'b' disabled
  unsigned char enabled;  //Enabled byte '/0' disabled  'a' enabled
  unsigned char device_name[MAX_DEVICE_NAME_LENGTH];
  unsigned char var_name   [MAX_NAME_LENGTH];
  unsigned char value      [MAX_VAR_LENGTH];
} devDataPacket;
#pragma pack()

#define DPACKETSIZE sizeof(devDataPacket)

/****************************************/
/*some other structures used by several */
/* classes .....                        */
/****************************************/


//structure to hold all plot Options
//it is passed to the Plotwidget to initialize it
//used in list.cpp and plot.cpp
typedef struct {
  int pminx;
  int pminy;
  int pleftborder;
  int pupperborder;
  int pdefsizex;
  int pdefsizey;
  int xticks;
  int yticks;
  int ticksize;
  int tickpos;
  int tickposx;
  int as_interval;
  double yupper;
  double ylower;
  bool hgridlines;
  bool vgridlines;
  bool autoscale;
  } PlotSettings;
  
#endif
