/*****************************************************************************
**$Id: plot.cpp,v 1.2 1999/02/21 18:49:16 rosen Exp rosen $
**$Log $
******************************************************************************

**                            copyright by Christian Rosen,
**Max Born Institute for nonlinear Optics and short time spectroscopy, MBI
**send bug reports and whatever to : rosen@mbi-berlin.de
****************************************************************************/
/*****************************************************************************
**                                    MAX ver 2.0
*****************************************************************************/
/*
 *Plot widget to plot measured data for max data acquisition software
 */
//STL includes
#include <vector>

//qt includes
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "plot.h"
#include "qtincludes.h"
#include <qpainter.h>
#include <qpaintdevicemetrics.h>


//local includes
#include "definitions.h"
#include "error.hh"

//pixmap incudes
#include "Pixmaps/filesave.xpm"
#include "Pixmaps/fileprint.xpm"
//#include "Pixmaps/skullmini.xpm"
#include "Pixmaps/as.xpm"

#ifndef XSize
#define XSize 200
#define YSize 150
#endif

/************************************
 * Options widget to set y - scales *
 ***********************************/
ScaleW::ScaleW(char *sname,double high,double low) {
  setCaption("Y-Scale");	
  /*set buttons & text for Y-Scale options */
  label1    = new QLabel(this,"bla");
  label1   -> setText("Upper Limit ");
  limEdit  = new QLineEdit(this,"YScale");
  limEdit-> setMaxLength(10);
  headline  = new QLabel(this,"headline");
  headline -> setFont(QFont("Helvetica",14));
  headline -> setText(sname);
  /*set buttons and text for polld options*/
  label2    = new QLabel(this,"headline2");
  label2    -> setText("Lower Limit : ");
  limEdit2 = new QLineEdit(this,"bla");
  limEdit2-> setMaxLength(10);

  /*now the rest*/
  ok        = new QPushButton(this,"ok");
  ok       -> setDefault(TRUE);
  cancel    = new QPushButton(this,"cancel");
  ok       -> setText("o.k.");
  cancel   -> setText("cancel");
  connect(ok,SIGNAL(clicked()),SLOT(okpressed()));
  connect(cancel,SIGNAL(clicked()),SLOT(cancelpressed()));

  setFixedSize(XSize,YSize+20);//window has fixed size -> small
  label1    -> setGeometry(30,30,XSize,20);
  headline  -> setGeometry(50,10,XSize-40,20);
  limEdit   -> setGeometry(60,50,75,20);
  label2    -> setGeometry(50,70,XSize-40,20);
  limEdit2  -> setGeometry(60,90,75,20);
  ok        -> setGeometry(25,120,50,30);
  cancel    -> setGeometry(125,120,50,30);
  //
  //set numbers in QlineEdits
  //
  QString str;
  str.sprintf("%f",high);       //copy int to string
  limEdit -> setText(str); //setText in line Edit Window
  str.sprintf("%f",low);
  limEdit2 -> setText(str);
}
void ScaleW:: okpressed() {
  QString str,str2;
  str = limEdit  -> text();
  str2 =limEdit2 -> text();
  if (( str.toDouble()) >= 0)
    emit(SIG_setlimits((double) str.toDouble(),
		       (double) str2.toDouble() )); //call scheduler routine to set the value
  hide();
}
void ScaleW :: cancelpressed() {
 hide();
}
const char * fileSaveText = "Click this button to save the data currently visible"
" in this plot window."
"You will be prompted for a file name.\n\n"
"You can also select the Save command from the File menu in the MainWindow "
"but that will save all data in memory";
const char * filePrintText = "Click this button to print the data currently"
"visible in the plot window ";

/*******************************************
 * PLOT WIDGET CLASS                       *
 * this class plots arrays,ints and doubles*
 * into user defined coordinate system     *
 ******************************************/

Plot::Plot(QWidget *parent,const char *name,char *devname, char *varname,int chan,int m)
  : QMainWindow( 0, "example application main window" )//QWidget(parent, name)
{
  printer = new QPrinter;
  printer->setMinMax( 1, 10 );
  QPixmap openIcon, saveIcon, printIcon, asIcon,killIcon;

  va_name = new char[sizeof(varname)];
  ASSERT (va_name != 0);
  strncpy(va_name,varname,sizeof(varname));
  upd = TRUE;
  monitor_mode = m;
  dmax   = 0;
  dmin   = 0;
  min    = 0;
  max    = 0;
  array_length = 0;
  arrays    = 0; //flag indicates wether arrays (and type) or int to plot
  l_plotlim = 0;
  draw_axis = TRUE;
  yhigh     = Y_UPPER_LIMIT;
  ylow      = Y_LOWER_LIMIT;
  linestyle = SOLIDLINE;
  xlow      = 0;
  xhigh     = chan;
  if (xhigh < 50)
    xhigh = 50;
  if (monitor_mode)
    xhigh = MON_MODE_CHANNELS;
  axiscolor = AXISCOLOR;
  linecolor = LINECOLOR;
  datacolor = LINECOLOR;
  arraylinecolor = ARRAYLINECOLOR;
  background= BACKGROUND;
  labelcolor= LABELCOLOR;
  autoscale = AUTOSCALE;
  as_interval = AS_INTERVAL; //intervall for autoscaling
  vgrid     = VGRIDLINES;
  hgrid     = HGRIDLINES;
  as_counter= 0;
  las_counter  =0;
  xticks       = X_TICKS;
  yticks       = Y_TICKS;
  PLeftBorder  = P_LEFT_BORDER;
  PUpperBorder = P_UPPER_BORDER;
  ticksize     = TICKSIZE;
  tickpos      = TICKPOS;
  tickposx     = TICKPOS_X;

  // first define the layout of the widget
  setMinimumSize(P_MIN_SIZE_X,P_MIN_SIZE_Y);
  setCaption(devname);
  this -> setMouseTracking(TRUE);

  //Set up toolbar with icons

  fileTools = new QToolBar( this, "file operations" );
  saveIcon = QPixmap( filesave );
  QToolButton * fileSave = new QToolButton( saveIcon, "Save File", 0,
					      this, SLOT(save()),
					      fileTools, "save file" );
  printIcon = QPixmap( fileprint );
  QToolButton * filePrint = new QToolButton( printIcon, "Print File", 0,
		 			     this, SLOT(print()),
					     fileTools, "print file" );
  asIcon  = QPixmap( as_xpm );
  QToolButton * autoScale = new QToolButton( asIcon, "Toggle Autoscaling", 0,
					     this, SLOT(toggleAS()),
					     fileTools, "Toggle Autoscaling" );
  (void)QWhatsThis::whatsThisButton( fileTools );


  QWhatsThis::add( fileSave,  fileSaveText,  FALSE );
  QWhatsThis::add( filePrint, filePrintText, FALSE );
  // QWhatsThis::add( killIcon,  filePrintText, FALSE );

  //set up Menu
  QPopupMenu *options = new QPopupMenu;
  CHECK_PTR( options );
  options->insertItem("Set Y-Scale",this,SLOT (setscales ())  );
  options->insertItem("update on",  this,SLOT (updateOn  ())  );
  options->insertItem("update off", this,SLOT (updateOff ())  );
  options->insertItem("ascale_on",  this,SLOT (autoscaleOn()) );
  options->insertItem("ascale_off", this,SLOT (autoscaleOff()));
  options->insertItem("hgrid_on",   this,SLOT (hgridOn  ())   );
  options->insertItem("hgrid_off",  this,SLOT (hgridOff ())   );
  options->insertItem("vgrid_on",   this,SLOT (vgridOn  ())   );
  options->insertItem("vgrid_off",  this,SLOT (vgridOff ())   );
  menu = new QMenuBar( this );
  CHECK_PTR( menu );
  menu ->insertItem( "&options",   options );

  Title  = new QLabel(this, "chn_label" );
  Title  -> setMinimumHeight(20);//,Title -> sizeHint().width());
  Title  -> setAlignment(AlignHCenter);
  Title  -> setText(varname);
  Title  -> setFont(QFont("Helvetica",14,QFont::Bold));
  XLabel = new QLabel(this);
  XLabel -> setFixedHeight(20);//XLabel -> sizeHint().height());
  XLabel -> setText("channels");
  XLabel -> setAlignment(AlignHCenter);
  XLabel -> setFont(QFont("Helvetica",14,QFont::Bold));
  xval   = new QLabel(this);
  xval 	 ->setMinimumHeight(15);
  xval 	 -> setAlignment(AlignHCenter);
  xval   ->setFrameStyle( QFrame::Box | QFrame::Sunken );
  yval 	 = new QLabel(this);
  yval 	 ->setMinimumHeight(15);
  yval 	 ->setAlignment(AlignHCenter);
  yval->setFrameStyle( QFrame::Box | QFrame::Sunken );
  as     = new QLabel(this);
  as     -> setMinimumHeight(15);
  as	 -> setAlignment(AlignHCenter);
  if (autoscale) {
    as -> setText("as enabled");
  }
  else {
    as -> setText("as disabled");
  }
  SW = new ScaleW(va_name,yhigh,ylow);
  //connect signal from Y-scale editor Widget
  connect(SW, SIGNAL(SIG_setlimits(double,double)),
	  this,SLOT ( SLOT_set_scales(double, double )) );
}
//
//destructor
//
Plot::~Plot(void) {
  //  delete va_name;
  //  delete SW;
}
//
//public SLOTS for menubar
//
void Plot::setscales()    {
   SW -> show();
}
void Plot::updateOn()     { upd = TRUE;                 }
void Plot::updateOff()    { upd = FALSE;                }
void Plot::autoscaleOn()  { autoscale = TRUE; draw_axis = TRUE; update(); }
void Plot::autoscaleOff() { autoscale = FALSE;update(); }
void Plot::toggleAS() {
  if (autoscale) {
    autoscale = FALSE;
    update();
  }
  else {
    autoscale = TRUE;
    draw_axis = TRUE;
    update();
  }
}
void Plot::hgridOff()     { hgrid = FALSE;    update(); }
void Plot::hgridOn()      { hgrid = TRUE;     update(); }
void Plot::vgridOn()      { vgrid = TRUE;     update(); }
void Plot::vgridOff()     { vgrid = FALSE;    update(); }

//
//SLOTS from called from Set Scales Widget
void Plot::SLOT_set_scales(double yh,double yl) {
  printf("yhigh %f\n",yh);
  printf("ylow %f\n",yl);
  yhigh = yh; ylow = yl;
  draw_axis = TRUE;
  update();
}
//
//plot routines :
//call this routines to add another point to the plot
//
/******************************
 *plot point of type int      *
 *****************************/
void Plot::plotPoint(int x,int y) {
  type = 0;
  draw_axis = FALSE; //never redraw the axis until loop comes and plot has to be erased
  point vpoint; //new structure to accomodate x and y value
  if (x==1) { //first channel allocate space for the vector
  }
  if (monitor_mode && x >= MON_MODE_CHANNELS-1) {
    x =1 ; //reset x
    draw_axis = TRUE;
    plotVec.erase(plotVec.begin(),plotVec.end()); //monitor mode, clear database
  }
  vpoint.x = x;			//copy values
  vpoint.y = (double) y;        //copy values
  plotVec.push_back(vpoint); //append plot data to the end of vector
  if (autoscale) {
    //calculate maximum and minimum first
    if (y >= max)
      max = y;
    if (y <= min)
      min = y;
    if (y >= yhigh ) {
      as_counter ++;
      if (as_counter == as_interval) {
	as_counter = 0;
	yhigh =max; //reset upper limit to new maximum
	max =0;
	update();
      }
    }
    else if (y < ylow ) {
      if (las_counter++ == as_interval) {
	as_counter = 0;
	ylow =min; //reset lower limit to new minimum
	min =0;
	update();	
      }
    }
    else {
      las_counter =0;
      as_counter =0; //only autoscale if as_intervall points beyond limits
    }
  }
}
/******************************
 *plot point of type double   *
 *****************************/
void Plot::plotPoint(int x,double y) {
  type = 1; //double var
  draw_axis = FALSE; //never redraw the axis until loop comes and plot has to be erased
  point vpoint;
  if (monitor_mode && x >= MON_MODE_CHANNELS-1) {
    x =1 ; //reset x
    draw_axis = TRUE;
    plotVec.erase( plotVec.begin(), plotVec.end());
    update();
  }
  vpoint.x = x;			//copy values
  vpoint.y = (double) y;        //copy values
  plotVec.push_back(vpoint); 	//append plot data to the end of vector
  if (autoscale) {  		//check for limits and modifiy if neccessary
    //calculate maximum and minimum first
    if (y > dmax)
      dmax = y;
    if (y < dmin)
      dmin = y;
    if (y > yhigh ) {
      as_counter ++;
      if (as_counter == as_interval){
	as_counter = 0; //reset counter for next run
	yhigh =dmax; //reset upper limit to new maximum
	dmax = 0;
	draw_axis = TRUE; //clear widget
	update();
      }
    }
    else if (y < ylow ) {
      las_counter ++;
      if (las_counter == as_interval) {
	as_counter = 0; //reset counter for next run
	ylow =dmin; //reset lower limit to new minimum
	dmin =0;
	draw_axis = TRUE;
	update();
      }
    }
    else {
      las_counter = 0;
      as_counter =0; //only autoscale if as_intervall points have beyond limits
    }
  }
}
/************************************************/
/* plot array of integer, erase old array first */
/************************************************/
void Plot::plotPoint(int chan,int array[],int length) {

}
/************************************************/
/* plot array of double erase old array first   */
/*also take care of proper scaling of values   */
/**************************************** ********/
void Plot::plotPoint(int chan,double array[],int length) {

}
/************************************************/
/* plot vector  of int  erase old vector  first */
/*also take care of proper scaling of values    */
/************************************************/
void Plot::plotPoint(int chan,vector<int> *iVPtr,int length) {
  type      = 1;
  arrays    = 1;	
  iVecPtr   = iVPtr;
  //Copy pointer to this vector, deep copy is made during refresh cyclke
  //Now check for autoscaling
  //and set plot limits
  xlow         = 0;
  xhigh        = iVecPtr->size();
  array_length = iVecPtr->size();
  if (xhigh < MIN_ARRAY_LENGTH)
    xhigh = MIN_ARRAY_LENGTH;
  draw_axis = TRUE;
  datacolor = yellow;//arraylinecolor;
  linecolor = arraylinecolor;
  /* double maximum =0;
  double minimum =0;
   if (autoscale) {  //here we use an iterator to "travel" over the vector
    for (vector<int>::iterator pIter=iVecPtr -> begin();pIter < iVecPtr -> end();pIter++) {
      if (*pIter > maximum)
	maximum = *pIter;
      if (*pIter < minimum)
	minimum = *pIter;
      if (*pIter > yhigh ) {
	as_counter ++;
	if (as_counter == as_interval)
	  yhigh =maximum; //reset upper limit to new maximum
      }
      else if (*pIter < ylow ) {
	las_counter ++;
	if (las_counter == as_interval)
	  ylow =minimum; //reset lower limit to new minimum
      }
      else {
	las_counter = 0;
	as_counter =0; //only scale if as_interval points have beyond limits}
      }
    }
  }*/
}
void Plot::plotPoint(int chan,vector<double> *dVPtr,int length) {
  type =2;
  arrays=2;
  dVecPtr = dVPtr;
}
void Plot::del() {
  printf("killing ...");
}
//draw contents of plot list
void Plot::DrawPlot( QPainter *p) {
  unsigned int pos;
  int index = 0;
  if (ylow <  0 ) 		// move x-axis to the middle
    offset = (fabs(ylow));
  xscale = (double ) pXsize/(xhigh-xlow);
  yscale = (double ) pYsize/(yhigh-ylow);
  p -> setPen (datacolor);	//datacolor/*linecolor*/ );
  if (plotVec.size()>0 && !array_length) {  			//plot scalar
    for (pos=0;pos < plotVec.size();pos++) {
      if (index++ == 0) {
	p -> moveTo((int ) ( pXpos+ xscale*plotVec[pos].x ),
		    (int ) ( pYpos+ pYsize-yscale*(offset+plotVec[pos].y)) );
      }
      if (linestyle == SOLIDLINE)
	p -> lineTo((int ) ( pXpos+xscale*plotVec[pos].x),
		    ( int )	( pYpos+ pYsize- yscale*(offset+plotVec[pos].y ) ) );
      else
	p -> drawPoint(( int ) (pXpos+ xscale*plotVec[pos].x),
		       ( int ) (pYpos+ pYsize- yscale*(offset + plotVec[pos].y) ) );
    }
  }
  else {
    if (array_length) { //now take the plot data from the real database
      for (pos =0;pos < plotVec.size();pos++) {
	if (index++ == 0) {
	  p -> moveTo((int ) ( pXpos+ xscale*pos ),
		      (int ) ( pYpos+ pYsize-yscale*(offset+plotVec[pos].y) ) );
	}
	if (linestyle == SOLIDLINE)
	  p -> lineTo((int ) ( pXpos+xscale*pos ),
		      ( int )	( pYpos+ pYsize- yscale* (offset+plotVec[pos].y ) ) );
	else
	  p -> drawPoint(( int ) (pXpos+ xscale*pos),
			 ( int ) (pYpos+ pYsize- yscale*(offset +plotVec[pos].y) ) );
      }	
    }	
  }///reset viewport back to full Window Size
  p -> setViewport(0,0,width(),height() );
  p -> scale(1,1);
}
void Plot::DrawAxes( QPainter *p ) {
  QString str;
  int i,xspacing,yspacing;
  int xpos,ypos;
  double xsteps;
  double ysteps;
  if (xticks >1)
    xspacing =  pXsize/(xticks);
  else
    xspacing = pXsize/2;

  if (yticks >1)
    yspacing =  pYsize/(xticks);
  else
    yspacing = pYsize/2;
  p -> setPen (background);
  p -> fillRect( pXpos, pYpos,pXsize, pYsize,black);
  p -> setPen (red);
  p -> drawRect( pXpos, pYpos,pXsize, pYsize);
  //draw yticks
  xpos = pXpos-ticksize;

  QPen pen( linecolor, 1,DashLine );
  for (i = 0;i < yticks;i++) {
    ypos = pYpos+i*yspacing;
    p -> drawLine( xpos,ypos,pXpos,ypos);
    p -> drawLine( pXpos+pXsize,ypos,pXpos+pXsize+ticksize,ypos);
    p -> setPen(pen);
    if ( i && hgrid)
	p -> drawLine( xpos,ypos,pXpos+pXsize,ypos);
  }
  p -> setPen(axiscolor);
  //draw x_ticks
  ypos = pYpos+pYsize;
  for (i = 0;i < xticks;i++) {
    xpos = pXpos+i*xspacing;
    p -> drawLine( xpos,ypos,xpos,ypos+ticksize);
    p -> drawLine( xpos,ypos-pYsize,xpos,ypos-pYsize-ticksize);
    p -> setPen(pen);
    if ( i && vgrid)
      p -> drawLine( xpos,ypos,xpos,ypos-pYsize-ticksize);
  }
  //now add numbers at y_axis
  p->setPen( labelcolor );
  xpos = pXpos-ticksize-tickpos;
  if (yticks >1)
    ysteps = (yhigh-ylow)/yticks;
  else
    ysteps = (yhigh-ylow)/2;

  for   (i = 0;i <= xticks;i++) {
    ypos =  pYpos+pYsize-i*yspacing;
    if (fabs(yhigh) < 1)
      str.sprintf("%.2G",ylow+i*ysteps);
    else
      str.sprintf("%.0f",ylow+i*ysteps);
      p -> drawText(xpos,ypos,str);
  }
  //now add numbers at x_axis
  p->setPen( labelcolor );

  ypos = pYpos+pYsize+ticksize+tickposx;
  if (xticks >1)
    xsteps = (xhigh-xlow)/xticks;
  else
    xsteps = (xhigh-xlow)/2;

  for   (i = 0;i <= xticks;i++) {
    xpos =  pXpos+i*xspacing;
    if (fabs(xhigh) < 1)
      str.sprintf("%.2G",xlow+i*xsteps);
    else
      str.sprintf("%.0f",xlow+i*xsteps);
    p -> drawText(xpos,ypos,str);
  }
}
void Plot::setLinestyle(int style )
{
  linestyle = style;
}
void Plot::setColors(QColor line, QColor backgd, QColor axis)
{
  axiscolor = axis;
  linecolor = line;
  background= backgd;
}

void Plot::resizeEvent(QResizeEvent * ) {
  Title  -> setGeometry(width()/2-Title-> width()/2,menu -> height(),200,20);
  XLabel -> setGeometry(width()/2-XLabel->width()/2,
			height()-XLabel -> height(),200,20);
  xval   ->  setGeometry(0,height()-15-xval -> frameWidth()*2,
			 60+xval -> frameWidth()*2,
			 15+xval -> frameWidth()*2);
  yval   ->  setGeometry(xval -> width(),height()-15-yval -> frameWidth()*2,
			 120+yval -> frameWidth()*2,
			 15+yval -> frameWidth()*2);
  as     ->  setGeometry(width()-as-> width(),menu -> height(),
			 120,15);
  pXpos =  PLeftBorder;
  pYpos =  PUpperBorder+menu -> height();
  pXsize = width()-2*PLeftBorder;
  pYsize = height()-PUpperBorder-menu -> height()- PUpperBorder;
  QPainter paint;
  paint.begin( this );                    // start painting widget
  DrawAxes( &paint );
  DrawPlot( &paint );
  if (autoscale)
    as -> setText("autoscale on");
  else
    as -> setText("autoscale off");
  paint.end();
}
void Plot::paintEvent(QPaintEvent *){
  Title  -> setGeometry(width()/2-Title-> width()/2,menu -> height(),200,20);
  XLabel -> setGeometry(width()/2-XLabel->width()/2,
			height()-XLabel -> height(),200,20);
  pXpos =  PLeftBorder;
  pYpos =  PUpperBorder+menu -> height();
  pXsize = width()-2*PLeftBorder;
  pYsize = height()-PUpperBorder-menu -> height()- PUpperBorder;
  QPainter paint;

  paint.begin( this );                    // start painting widget
  if (upd) {
    if (draw_axis)
      DrawAxes( &paint );
  }
  DrawPlot( &paint );
  paint.end();
  draw_axis = TRUE;	
  l_plimit = 0;
  if (autoscale)
    as -> setText("autoscale on");
  else
    as -> setText("autoscale off");
}
void Plot::mouseMoveEvent(QMouseEvent *mouse) {
  QString str;
  if (mouse -> x() > pXpos && mouse -> x() < pXpos+pXsize &&
      mouse -> y() > pYpos && mouse -> y() < pYpos+pYsize) {
    setCursor(crossCursor);
    xscale = (double )  pXsize/(xhigh-xlow);
    yscale = (double ) pYsize/(yhigh-ylow);
    str.sprintf("x=%5.3f",1/xscale*(mouse -> x()-pXpos) );
    xval -> setText(str);
    str.sprintf("y=%5.3f",-1*(offset+1/yscale*(mouse -> y()-(pYpos+pYsize))));
    yval -> setText(str);
  }
  else
    setCursor(ArrowCursor);
}
//
//this functions receives the structure PlotSettings (-> packet.hpp)
//from main.cpp to initialize the Plotwidget
//
void Plot::SetAllOptions(PlotSettings *Ps)
{
  xticks      = Ps -> xticks;
  yticks      = Ps -> yticks;
  PLeftBorder = Ps -> pleftborder;
  PUpperBorder= Ps -> pupperborder;
  ticksize    = Ps -> ticksize;
  tickpos     = Ps -> tickpos;
  tickposx    = Ps -> tickposx;
  as_interval = Ps -> as_interval;
  yhigh       = Ps -> yupper;
  ylow        = Ps -> ylower;
  autoscale   = Ps -> autoscale;
  vgrid       = Ps -> vgridlines;
  hgrid       = Ps -> hgridlines;
  //recalculate all values
  pXpos =  PLeftBorder;
  pYpos =  PUpperBorder+menu -> height();
  pXsize = width()-2*PLeftBorder;
  pYsize = height()-PUpperBorder-menu -> height()- PUpperBorder;
  if (autoscale)
    as -> setText("autoscale on");
  else
    as -> setText("autoscale off");
  update();
}
//
//this function is called after all plot data has been passed to this widget
//it forces Qt to update the widget and show the data
//this avoids that data is plotted before all data has arrived
//the function only replots the area that has been drawn into, e.g.
//the area where new data has been added
void Plot::refresh(void) {
  double maximum = 0;
  double minimum = 0;
  int x= 0;
  pXpos =  PLeftBorder;
  pYpos =  PUpperBorder+menu -> height();
  pXsize = width()-2*PLeftBorder;
  pYsize = height()-PUpperBorder-menu -> height()- PUpperBorder;
  xscale = (double ) pXsize/(xhigh-xlow);
  yscale = (double ) pYsize/(yhigh-ylow);
  l_plimit = last_x; //don't replot old values already visible
  /**
   **if array in database copy it now from main into local database
   **this is neccessary in monitor mode
   **-> no main database for measured values exists
   **hence iptr and dptr point to nowhere
   */
  if (array_length) {
    plotVec.erase(plotVec.begin(),plotVec.end()); 	//erase vector before appending new stuff
    switch(arrays ) { 	//check for type of pointer	
    case 1:  		//int //pointer to last element is still valid
      for (vector<int>::iterator pIter=iVecPtr -> begin();pIter < iVecPtr -> end();pIter++) {
	point vpoint;
	vpoint.x = x++;
	vpoint.y = (double ) *pIter; //Iterator points to element itself
	plotVec.push_back(vpoint);
	if (autoscale) {  //here we use an iterator to "travel" over the vector
	  //    for (vector<int>::iterator pIter=iVecPtr -> begin();pIter < iVecPtr -> end();pIter++) {
	  if (*pIter > maximum)
	    maximum = *pIter;
	  if (*pIter < minimum)
	    minimum = *pIter;
	  if (*pIter > yhigh ) {
	    as_counter ++;
	    if (as_counter == as_interval)
	      yhigh =maximum; //reset upper limit to new maximum
	  }
	  else if (*pIter < ylow ) {
	    las_counter ++;
	    if (las_counter == as_interval)
	      ylow =minimum; //reset lower limit to new minimum
	  }
	  else {
	    las_counter = 0;
	    as_counter =0; //only scale if as_interval points have beyond limits}
	  }
	}
      }
    break;
    case 2: 		//double
      for (vector<double>::iterator dIter=dVecPtr -> begin();dIter < dVecPtr -> end();dIter++) {
	point vpoint;	
	vpoint.x = x++;
	vpoint.y = *dIter; //Iterator points to element itself
	plotVec.push_back(vpoint);
      	if (autoscale) {  //here we use an iterator to "travel" over the vector
	  if (*dIter > maximum)
	    maximum = *dIter;
	  if (*dIter < minimum)
	    minimum = *dIter;
	  if (*dIter > yhigh ) {
	    as_counter ++;
	    if (as_counter == as_interval)
	      yhigh =maximum; //reset upper limit to new maximum
	  }
	  else if (*dIter < ylow ) {
	    las_counter ++;
	    if (las_counter == as_interval)
	      ylow =minimum; //reset lower limit to new minimum
	  }
	  else {
	    las_counter = 0;
	    as_counter =0; //only scale if as_interval points have beyond limits}
	  }
	}
      }
    break;
    }
  }
  repaint(FALSE);
}
void Plot::save() {
  FILE *fp1;
  int ok=0;
  unsigned int pos;
  int result;
  QString f = QFileDialog::getSaveFileName( 0,"*.dat", this, "Save Current Plot" );

  if  ( !f.isEmpty() ) { //the user pressed ok
    if ((fp1 = fopen((const char *) f,"w")) == NULL ) {
     result=Ierror(E_OPEN,"routine to save data ","");
     result = 0;
     return;
     }
    QFileInfo fim(f); //check if file exists already
    if( fim.exists() ){
      switch ( QMessageBox::warning(this,"WARNING","File exists\noverwrite ?",
				     QMessageBox::No | QMessageBox::Default,
				     QMessageBox::Yes  | QMessageBox::Escape )) {
      case QMessageBox::Yes :
	ok =1;
	break;
      case QMessageBox::No :
	ok = 0;	
	break;
      }
    }	

    if (ok) {
      //now save the data in the plotwindow
      switch(type) {
      case 0 : //convert data back to integer
	for ( pos =0;pos < plotVec.size();pos++) {
	  printf("saving %s %d %d \n",(const char *) f,pos,(int) plotVec[pos].y);
	  fprintf(fp1,"%d %d\n",pos, (int ) plotVec[pos].y);
	}
	break;
      case 1 : //save as double
	for (pos =0;pos < plotVec.size();pos++) {
	  fprintf(fp1,"%d %G\n",pos, plotVec[pos].y);
	}
	break;
      }	
    printf("Max, Plot Window : Done saving %s \n",va_name);
    }
  }
}
void Plot::print() {
  //  const int MARGIN = 10;
  //int pageNo = 1;

  if ( printer->setup(this) ) {		// printer dialog
    printf("printing ..\n");
    QPainter p;
    p.begin( printer );			// paint on printer
    QPen pn=p.pen(); //set pen for this drawing
    pn.setWidth(4);
    QPaintDeviceMetrics metrics( printer ); // need width/height
    // of printer surface
    p.setWindow(20,20,pXsize+20,pYsize+20); //size of page to print on
    p.setViewport(0,0,metrics.width(),metrics.height());
    //printf("width %d height %d \n",metrics.height(),metrics.width());
    p.setFont( QFont("helvetica", 50,QFont::Bold));
    p.drawText(0,0, "Hello world");
    //
    DrawPlot(&p);
    // DrawAxes(&p);
    p.end();				// send job to printer
    printf("done..\n");
  }
  else {
    printf("abort ..\n");
  }	
}	

void Plot::kill() {
  ::exit(1);
}

