/**************************************************************************
 * $Id: priority.hpp,v 1.5 1999/02/21 18:48:11 rosen Exp rosen $
 *
 * Revision 1.01 1998/10/03 14:34:15  rosen
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **************************************************************************/

//header file for the class definition for the priority list
//the priority list conatins the names of the devices and their priority
//e.g. the roder they are called in
//the scheduler calls all devices according to their priority list


#ifndef PRIORITY_H
#define PRIORITY_H

#include <qwidget.h>
#include <qtimer.h>
#include <qimage.h>
#include <assert.h>
#include <stdio.h>
#include "error.hh"

class Pr_element {
  friend class Pr_list;
  private :
  Pr_element( char *name =0, int p=0, Pr_element *E = 0) {
     priority = p;
    next = E;
    device_name = new char[strlen(name)+1]; //reserve mem for dev_name
    assert(device_name != 0);               //not enough mem
    strcpy(device_name,name);               //copy dev_name into reserved mem of class
  }
  int   priority;
  char *device_name;
  Pr_element* next;
};

class Pr_list {                //this class creates the linked list for the priority numbers
  friend class Scheduler;
  friend class Data_list;
  public :
  Pr_list () { L = 0; };
  ~Pr_list() { Pr_kill_list(); };

  Pr_element * end();          //returns pointer to the end of the list
   Pr_element * exist(char *);    //return pointer to element char *dev_name in list or NULL
  void   Pr_append(char *,int);  //append to list
  void   Pr_sort();              //sort list according to priorities -> faster search useful ??
                               //at least it is  nicer to edit a sorted list on the screen
  int    N_Devices();		//Number of Devices in Priority list
  int    Max_Priority();         //return Maximum priority
  int    Pr_initlist(char *);    //create list from diskinitfile
  void   Pr_kill_list();         //remove complete list out of memory
  void   Pr_remove();
  void   Put_priority(int, int);//Put (prior, index)
  void 	 Sort_priority();   //put priorities in right order and check for double an missing occurences
  int    Pr_empty ();
  int    exist(int );             //returns 1 if int exist as priority
  int    Get_priority(char *);    //returns priority of device char* <device_name>
  int    Get_priority(int );      //returns priority of device i
  int    check4double();          //check for double priority
  bool   check4zerolist();	  //check wether all priorities are zero
  int    varcomp(Pr_element *,char *);//compares string with device_name in priority list
  char*  Get_device_name(int );
  char * Get_dev_name (int );  //get device name defined by priority
  void   Pr_printlist();
  int    Consistency();  //Check order of devices for consistency. Return = if o.k. or number of missing element
 
   /*network related functions only in ver 2.0 !!!*/
  void   ResetHelpPointer(void);         //reset help list pointer to beginning of list
  int     AdvanceHelpPointer(void);    //go to next element
  char*   GetNextName();                    //return device name pointed to by BufferPointer
  int     GetNextPriority();                 //return priority pointed to by BufferPointer
  /* end */
  private :
  Pr_element *L;
  Pr_element *Buffer_pointer,*BufferPointer;
};
#endif PRIORITY_H
